from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.utils import to_net_list
from typing import List


class TuneDataValues(NetWrapperBase):

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.TuneDataValues

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    @property
    def id(self) -> int:
        """
        Gets or sets the index number of the tune record
        """
        return self._get_wrapped_object_().ID

    @id.setter
    def id(self, value: int):
        """
        Gets or sets the index number of the tune record
        """
        assert type(value) is int
        self._get_wrapped_object_().ID = value

    @property
    def values(self) -> List[str]:
        """
        Gets or sets the array of tune data values for an instrument
        """
        return self._get_wrapped_object_().Values

    @values.setter
    def values(self, value: List[str]):
        """
        Gets or sets the array of tune data values for an instrument
        """
        assert type(value) is list
        value = to_net_list(value, str)
        self._get_wrapped_object_().Values = value




from fisher_py.data.business.sample_type import SampleType
from fisher_py.data.business.bracket_type import BracketType
from fisher_py.data.business.barcode_status_type import BarcodeStatusType
from fisher_py.data.business.spectrum_packet_type import SpectrumPacketType
from fisher_py.data.business.tolerance_mode import ToleranceMode
from fisher_py.data.business.trace_type import TraceType
from fisher_py.data.business.data_units import DataUnits
from fisher_py.data.business.generic_data_types import GenericDataTypes
from fisher_py.data.business.tune_data_values import TuneDataValues
from fisher_py.data.business.status_log_values import StatusLogValues
from fisher_py.data.business.header_item import HeaderItem
from fisher_py.data.business.log_entry import LogEntry
from fisher_py.data.business.instrument_data import InstrumentData
from fisher_py.data.business.mass_options import MassOptions
from fisher_py.data.business.range import Range
from fisher_py.data.business.chromatogram_trace_settings import ChromatogramTraceSettings
from fisher_py.data.business.instrument_selection import InstrumentSelection
from fisher_py.data.business.noise_and_baseline import NoiseAndBaseline
from fisher_py.data.business.mass_to_frequency_converter import MassToFrequencyConverter
from fisher_py.data.business.simple_scan import SimpleScan
from fisher_py.data.business.scan_statistics import ScanStatistics
from fisher_py.data.business.label_peak import LabelPeak
from fisher_py.data.business.run_header import RunHeader
from fisher_py.data.business.sample_information import SampleInformation
from fisher_py.data.business.cached_scan_provider import CachedScanProvider
from fisher_py.data.business.segmented_scan import SegmentedScan
from fisher_py.data.business.centroid_stream import CentroidStream
from fisher_py.data.business.scan import Scan
from fisher_py.data.business.reaction import Reaction
from fisher_py.data.business.chromatogram_signal_cls import ChromatogramSignal



import enum


class GenericDataTypes(enum.Enum):
    """
    enumeration for data type for the fields used by records of TuneData, StatusLog,
    TrailerExtra These are upper case names, so that they don't clash with standard
    type names.
    """
    NULL = 0
    CHAR = 1
    TRUEFALSE = 2
    YESNO = 3
    ONOFF = 4
    UCHAR = 5
    SHORT = 6
    USHORT = 7
    LONG = 8
    ULONG = 9
    FLOAT = 10
    DOUBLE = 11
    CHAR_STRING = 12
    WCHAR_STRING = 13



"""
A Python example program showing how to use RAWFileReader. More information on the RAWFileReader methods used
in this example and the other methods available in RAWFileReader can be found in the RAWFileReader user
documentation, that is installed with the RAWFileReader software.

The code of this example is based on the following GitHub repository: https://github.com/compomics/ThermoRawFileParser
It was added to provide users a way of doing what ThermoRawFileParser does in Python.
"""

from typing import List, Union

from numpy import float64
from fisher_py.data import Device
from fisher_py.data import ScanEvent
from fisher_py.data.filter_enums import MsOrderType, PolarityType, ScanDataType
from fisher_py.raw_file_reader import RawFileReaderAdapter
from fisher_py.data.business import Scan, Reaction, LogEntry
import os
import re


## Configuration (Use this section to select the input file and the output folder as well as setting other parameters)
RAW_FILE_PATH = r"my_file.raw"
OUTPUT_FOLDER = r"output_folder"
MS_LEVEL = { MsOrderType.Ms2 }
DISABLE_NATIVE_PEAK_SEARCHING = True
INCLUDE_REF_AND_EX_DATA = True
#--

# Constants
PROGRESS_STEP = 10
ZERO_DELTA = 0.0001
PRECURSOR_MZ_DELTA = 0.0001
DEFAULT_ISOLATION_WINDOW_LOWER_OFFSET = 1.5
DEFAULT_ISOLATION_WINDOW_UPPER_OFFSET = 2.5
FILTER_STRING_ISOLATION_MZ_PATTERN = 'ms2 (.*?)@'
_isolation_mz_to_precursor_scan_number_mapping = dict()
_precursor_ms1_scan_number = 0
_last_scan_progress = 0


def construct_spectrum_title(instrument_type: int, instrument_number: int, scan_number: int) -> str:
    return f'controllerType={instrument_type} controllerNumber={instrument_number} scan={scan_number}'


def construct_precursor_reference(ms_order: MsOrderType, scan_number: int, scan_event: ScanEvent) -> str:
    global _isolation_mz_to_precursor_scan_number_mapping, _precursor_ms1_scan_number
    
    if ms_order == MsOrderType.Ms2:
        # Keep track of the MS2 scan number and isolation m/z for precursor reference
        result = re.match(str(scan_event), FILTER_STRING_ISOLATION_MZ_PATTERN) 
        if result:
            res = result.groups()[0]
            if res in _isolation_mz_to_precursor_scan_number_mapping:
                del _isolation_mz_to_precursor_scan_number_mapping[res]
            _isolation_mz_to_precursor_scan_number_mapping[res] = scan_number
            
        precursor_reference = construct_spectrum_title(Device.MS.value, 1, _precursor_ms1_scan_number)
    
    elif ms_order == MsOrderType.Ms3:
        precursor_scan_number = next(filter(lambda isolation_mz: isolation_mz in str(scan_event),  _isolation_mz_to_precursor_scan_number_mapping.keys()), None)
        if precursor_scan_number:
            precursor_reference = construct_precursor_reference(Device.MS.value, 1, _isolation_mz_to_precursor_scan_number_mapping[precursor_scan_number])
        else:
            raise ValueError(f"Couldn't find a MS2 precursor scan for MS3 scan {scan_event}")
        
    return precursor_reference


def get_reaction(scan_event: ScanEvent, scan_number: int) -> Reaction:
    try:
        order = scan_event.ms_order.value
        return scan_event.get_reaction(order - 2)
    except:
        print(f'No reaction found for scan {scan_number}')
        return None
    
    
class ScanTrailer(object):
    
    @property
    def length(self) -> int:
        return len(self._data)
    
    @property
    def labels(self) -> List[str]:
        return list(self._data.keys())
    
    @property
    def values(self) -> List[str]:
        return list(self._data.values())
    
    def as_bool(self, key: str) -> Union[bool, None]:
        if key in self._data:
            str_value = self._data[key].lower()
            
            if str_value in { 'on', 'true', 'yes' }:
                return True
            return False
        return None
    
    def as_double(self, key: str) -> float64:
        if key in self._data:
            try:
                return float64(self._data[key])
            except:
                pass
        return None
    
    def as_int(self, key: str) -> int:
        if key in self._data:
            try:
                return int(self._data[key])
            except:
                pass
        return None
    
    def as_positive_int(self, key: str) -> int:
        if key in self._data:
            try:
                result = int(self._data[key])
                return result if result >= 0 else None
            except:
                pass
        return None 
    
    def as_string(self, key: str) -> str:
        return self.get(key)
    
    def get(self, key: str) -> str:
        if key in self._data:
            try:
                return self._data[key]
            except:
                pass
        return None
    
    def has(self, key: str) -> bool:
        return key in self._data
    
    def __init__(self, trailer_data: LogEntry):
        self._data = {trailer_data.labels[i]: trailer_data.values[i] for i in range(trailer_data.length)}


def calculate_selected_ion_mz(reaction: Reaction, monoisotopic_mz: float64, isolation_width: float64) -> float64:
    selected_ion_mz = reaction.precursor_mass
    
    # take the isolation width from the reaction if no value was found in the trailer data
    if isolation_width is None or isolation_width < ZERO_DELTA:
        isolation_width = reaction.isolation_width
        
    isolation_width *= 0.5
    
    if monoisotopic_mz and monoisotopic_mz > ZERO_DELTA and abs(reaction.precursor_mass - monoisotopic_mz) > PRECURSOR_MZ_DELTA:
        selected_ion_mz = monoisotopic_mz
        
        # check if the monoisotopic mass lies in the precursor mass isolation window
        # otherwise take the precursor mass
        if isolation_width <= 2:
            if (selected_ion_mz < (reaction.precursor_mass - DEFAULT_ISOLATION_WINDOW_LOWER_OFFSET * 2)) or (selected_ion_mz > (reaction.precursor_mass + DEFAULT_ISOLATION_WINDOW_UPPER_OFFSET)):
                selected_ion_mz = reaction.precursor_mass
        elif (selected_ion_mz < (reaction.precursor_mass - isolation_width)) or (selected_ion_mz > (reaction.precursor_mass + isolation_width)):
            selected_ion_mz = reaction.precursor_mass
            
    return selected_ion_mz


if __name__ == '__main__':
    raw_file = RawFileReaderAdapter.file_factory(RAW_FILE_PATH)
    raw_file.select_instrument(Device.MS, 1)
    raw_file.include_reference_and_exception_data = INCLUDE_REF_AND_EX_DATA
    
    file_name = os.path.splitext(os.path.split(RAW_FILE_PATH)[-1])[0]
    OUTPUT_FILE = os.path.join(OUTPUT_FOLDER, f'{file_name}.mgf')

    # Get the first and last scan from the RAW file
    first_scan_number = raw_file.run_header_ex.first_spectrum
    last_scan_number = raw_file.run_header_ex.last_spectrum
    
    
    with open(OUTPUT_FILE, 'w') as mgf_file:
        for scan_number in range(first_scan_number, last_scan_number + 1):
            
            # report progress
            scan_progress = int(scan_number / (last_scan_number - first_scan_number + 1) * 100)
            if scan_progress % PROGRESS_STEP == 0:
                if scan_progress != _last_scan_progress:
                    print(f'{scan_progress} %')
                    _last_scan_progress = scan_progress
            
            # Get the scan from the RAW file
            scan: Scan = Scan.from_file(raw_file, scan_number)

            # Get the retention time
            retention_time = raw_file.retention_time_from_scan_number(scan_number)

            # Get the scan filter for this scan number
            scan_filter = raw_file.get_filter_for_scan_number(scan_number)
            
            # Get the scan event for this scan number
            scan_event = raw_file.get_scan_event_for_scan_number(scan_number)
            
            # Don't include MS1 spectra
            if scan_filter.ms_order in MS_LEVEL:
                reaction = get_reaction(scan_event, scan_number)
                
                mgf_file.write(f'BEGIN IONS\n')
                mgf_file.write(f'TITLE={construct_spectrum_title(Device.MS.value, 1, scan_number)}\n')
                
                mgf_file.write(f'SCANS={scan_number}\n')
                mgf_file.write(f'RTINSECONDS={(retention_time * 60)}\n')
                
                # Trailer extra data list
                trailer_data = ScanTrailer(raw_file.get_trailer_extra_information(scan_number))
                charge = trailer_data.as_positive_int('Charge State:')
                monoisotopic_mz = trailer_data.as_double('Monoisotopic M/Z:')
                isolation_width = trailer_data.as_double(f'MS{scan_filter.ms_order.value} Isolation Width:')
                
                if reaction:
                    selected_ion_mz = calculate_selected_ion_mz(reaction, monoisotopic_mz, isolation_width)
                    mgf_file.write(f'PEPMASS={selected_ion_mz}\n')
                    
                # Charge
                if charge:
                    # Scan polarity
                    polarity = '+' if scan_filter.polarity == PolarityType.Positive else '-'
                    mgf_file.write(f'CHARGE={charge}{polarity}\n')
                
                if not DISABLE_NATIVE_PEAK_SEARCHING:
                    # Check if the scan has a centroid stream
                    if scan.has_centroid_stream:
                        masses = scan.centroid_scan.masses
                        intensities = scan.centroid_scan.intensities
                    else:    # Otherwise take segmented (low res) scan data
                        segmented_scan = Scan.to_centroid(scan) if scan_event.scan_data == ScanDataType.Profile else scan.segmented_scan
                        masses = segmented_scan.positions
                        intensities = segmented_scan.intensities
                else:    # Use the segmented data as is
                    masses = scan.segmented_scan.positions
                    intensities = scan.segmented_scan.intensities
                    
                if masses and len(masses):
                    masses, intensities = [l for l in zip(*sorted(zip(masses, intensities)))]                    
                    
                    for i in range(len(masses)):
                        mgf_file.write(f'{masses[i]:.5f} {intensities[i]:.3f}\n')
                        
                    mgf_file.write('END IONS\n')
                    



from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data import Device


class InstrumentSelection(NetWrapperBase):
    """
    Defines which instrument is selected in a file.
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.InstrumentSelection

    def __init__(self, instrument_index: int, device_type: Device):
        super().__init__()
        assert type(instrument_index) is int
        assert type(device_type) is Device
        self._wrapped_object = self._wrapped_object(instrument_index, device_type.value)

    @property
    def instrument_index(self) -> int:
        """
        Gets the Stream number (instance of this instrument type). Stream numbers start
        from 1
        """
        return self._get_wrapped_object_().InstrumentIndex

    @property
    def device_type(self) -> Device:
        """
        Gets the Category of instrument
        """
        return Device(self._get_wrapped_object_().DeviceType)


from __future__ import annotations
from typing import Any
from fisher_py.exceptions import CoreException


class NetWrapperBase(object):

    _wrapped_type = None

    def __init__(self):
        self._wrapped_object = None

    def _get_wrapped_object_(self) -> Any:
        if self._wrapped_object is None:
            raise CoreException('This object was not initialized properly.')

        return self._wrapped_object
    
    @classmethod
    def _get_wrapper_(cls: type, net_obj: Any) -> NetWrapperBase:
        wrapper_obj = cls()
        wrapper_obj._wrapped_object = net_obj
        return wrapper_obj



from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.utils import to_net_list
from typing import List


class StatusLogValues(NetWrapperBase):
    """
    Stores one record of status log values
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.StatusLogValues

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    @property
    def retention_time(self) -> float:
        """
        Gets or sets the RetentionTime for this log entry
        """
        return self._get_wrapped_object_().RetentionTime

    @retention_time.setter
    def retention_time(self, value: float):
        """
        Gets or sets the RetentionTime for this log entry
        """
        assert type(value) is float
        self._get_wrapped_object_().RetentionTime = value

    @property
    def values(self) -> List[str]:
        """
        Gets or sets the array of status log values
        """
        return self._get_wrapped_object_().Values

    @values.setter
    def values(self, value: List[str]):
        """
        Gets or sets the array of status log values
        """
        assert type(value) is list
        value = to_net_list(value, str)
        self._get_wrapped_object_().Values = value




from __future__ import annotations
from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data.business import SpectrumPacketType


class ScanStatistics(NetWrapperBase):
    """
    Summary information about a scan.
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.ScanStatistics

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    @property
    def wavelength_step(self) -> float:
        """
        Gets or sets the wave length step.
        """
        return self._get_wrapped_object_().WavelengthStep

    @wavelength_step.setter
    def wavelength_step(self, value: float):
        """
        Gets or sets the wave length step.
        """
        assert type(value) is float
        self._get_wrapped_object_().WavelengthStep = value

    @property
    def absorbance_unit_scale(self) -> float:
        """
        Gets or sets the absorbance unit scale.
        """
        return self._get_wrapped_object_().AbsorbanceUnitScale

    @absorbance_unit_scale.setter
    def absorbance_unit_scale(self, value: float):
        """
        Gets or sets the absorbance unit scale.
        """
        assert type(value) is float
        self._get_wrapped_object_().AbsorbanceUnitScale = value

    @property
    def is_uniform_time(self) -> bool:
        """
        Gets or sets a value indicating whether is uniform time.
        """
        return self._get_wrapped_object_().IsUniformTime

    @is_uniform_time.setter
    def is_uniform_time(self, value: bool):
        """
        Gets or sets a value indicating whether is uniform time.
        """
        assert type(value) is bool
        self._get_wrapped_object_().IsUniformTime = value

    @property
    def frequency(self) -> float:
        """
        Gets or sets the frequency.
        """
        return self._get_wrapped_object_().Frequency

    @frequency.setter
    def frequency(self, value: float):
        """
        Gets or sets the frequency.
        """
        assert type(value) is float
        self._get_wrapped_object_().Frequency = value

    @property
    def is_centroid_scan(self) -> bool:
        """
        Gets or sets a value indicating whether this scan contains centroid data (else
        profile)
        """
        return self._get_wrapped_object_().IsCentroidScan

    @is_centroid_scan.setter
    def is_centroid_scan(self, value: bool):
        """
        Gets or sets a value indicating whether this scan contains centroid data (else
        profile)
        """
        assert type(value) is bool
        self._get_wrapped_object_().IsCentroidScan = value

    @property
    def segment_number(self) -> int:
        """
        Gets or sets the time segment number for this event
        """
        return self._get_wrapped_object_().SegmentNumber

    @segment_number.setter
    def segment_number(self, value: int):
        """
        Gets or sets the time segment number for this event
        """
        assert type(value) is int
        self._get_wrapped_object_().SegmentNumber = value

    @property
    def scan_event_number(self) -> int:
        """
        Gets or sets the event (scan type) number within segment
        """
        return self._get_wrapped_object_().ScanEventNumber

    @scan_event_number.setter
    def scan_event_number(self, value: int):
        """
        Gets or sets the event (scan type) number within segment
        """
        assert type(value) is int
        self._get_wrapped_object_().ScanEventNumber = value

    @property
    def scan_number(self) -> int:
        """
        Gets or sets the number of the scan
        """
        return self._get_wrapped_object_().ScanNumber

    @scan_number.setter
    def scan_number(self, value: int):
        """
        Gets or sets the number of the scan
        """
        assert type(value) is int
        self._get_wrapped_object_().ScanNumber = value

    @property
    def number_of_channels(self) -> int:
        """
        Gets or sets the number of channels acquired in this scan, if this is UV or analog
        data,
        """
        return self._get_wrapped_object_().NumberOfChannels

    @number_of_channels.setter
    def number_of_channels(self, value: int):
        """
        Gets or sets the number of channels acquired in this scan, if this is UV or analog
        data,
        """
        assert type(value) is int
        self._get_wrapped_object_().NumberOfChannels = value

    @property
    def packet_count(self) -> int:
        """
        Gets or sets the Number of point in scan
        """
        return self._get_wrapped_object_().PacketCount

    @packet_count.setter
    def packet_count(self, value: int):
        """
        Gets or sets the Number of point in scan
        """
        assert type(value) is int
        self._get_wrapped_object_().PacketCount = value

    @property
    def start_time(self) -> float:
        """
        Gets or sets the time at start of scan (minutes)
        """
        return self._get_wrapped_object_().StartTime

    @start_time.setter
    def start_time(self, value: float):
        """
        Gets or sets the time at start of scan (minutes)
        """
        assert type(value) is float
        self._get_wrapped_object_().StartTime = value

    @property
    def tic(self) -> float:
        """
        Gets or sets the total Ion Current for scan
        """
        return self._get_wrapped_object_().TIC

    @tic.setter
    def tic(self, value: float):
        """
        Gets or sets the total Ion Current for scan
        """
        assert type(value) is float
        self._get_wrapped_object_().TIC = value

    @property
    def base_peak_mass(self) -> float:
        """
        Gets or sets the mass of largest peak in scan
        """
        return self._get_wrapped_object_().BasePeakMass

    @base_peak_mass.setter
    def base_peak_mass(self, value: float):
        """
        Gets or sets the mass of largest peak in scan
        """
        assert type(value) is float
        self._get_wrapped_object_().BasePeakMass = value

    @property
    def base_peak_intensity(self) -> float:
        """
        Gets or sets the intensity of highest peak in scan
        """
        return self._get_wrapped_object_().BasePeakIntensity

    @base_peak_intensity.setter
    def base_peak_intensity(self, value: float):
        """
        Gets or sets the intensity of highest peak in scan
        """
        assert type(value) is float
        self._get_wrapped_object_().BasePeakIntensity = value

    @property
    def short_wavelength(self) -> float:
        """
        Gets or sets the shortest wavelength in PDA scan
        """
        return self._get_wrapped_object_().ShortWavelength

    @short_wavelength.setter
    def short_wavelength(self, value: float):
        """
        Gets or sets the shortest wavelength in PDA scan
        """
        assert type(value) is float
        self._get_wrapped_object_().ShortWavelength = value

    @property
    def long_wavelength(self) -> float:
        """
        Gets or sets the longest wavelength in PDA scan
        """
        return self._get_wrapped_object_().LongWavelength

    @long_wavelength.setter
    def long_wavelength(self, value: float):
        """
        Gets or sets the longest wavelength in PDA scan
        """
        assert type(value) is float
        self._get_wrapped_object_().LongWavelength = value

    @property
    def low_mass(self) -> float:
        """
        Gets or sets the lowest mass in scan
        """
        return self._get_wrapped_object_().LowMass

    @low_mass.setter
    def low_mass(self, value: float):
        """
        Gets or sets the lowest mass in scan
        """
        assert type(value) is float
        self._get_wrapped_object_().LowMass = value

    @property
    def high_mass(self) -> float:
        """
        Gets or sets the highest mass in scan
        """
        return self._get_wrapped_object_().HighMass

    @high_mass.setter
    def high_mass(self, value: float):
        """
        Gets or sets the highest mass in scan
        """
        assert type(value) is float
        self._get_wrapped_object_().HighMass = value

    @property
    def spectrum_packet_type(self) -> SpectrumPacketType:
        """
        Gets the packet format used in this scan. (read only). Value can be set using
        the "PacketType" property.
        """
        return SpectrumPacketType(self._get_wrapped_object_().SpectrumPacketType)

    @property
    def packet_type(self) -> int:
        """
        Gets or sets the indication of data format used by this scan. See also SpectrumPacketType
        for decoding to an enum.
        """
        return self._get_wrapped_object_().PacketType

    @packet_type.setter
    def packet_type(self, value: int):
        """
        Gets or sets the indication of data format used by this scan. See also SpectrumPacketType
        for decoding to an enum.
        """
        assert type(value) is int
        self._get_wrapped_object_().PacketType = value

    @property
    def scan_type(self) -> str:
        """
        Gets or sets a String defining the scan type, for filtering
        """
        return self._get_wrapped_object_().ScanType

    @scan_type.setter
    def scan_type(self, value: str):
        """
        Gets or sets a String defining the scan type, for filtering
        """
        assert type(value) is str
        self._get_wrapped_object_().ScanType = value

    @property
    def cycle_number(self) -> int:
        """
        Gets or sets the cycle number. Cycle number used to associate events within a
        scan event cycle. For example, on the first cycle of scan events, all the events
        would set this to '1'. On the second cycle, all the events would set this to
        '2'. This field must be set by devices if supporting compound names for filtering.
        However, it may be set in all acquisitions to help processing algorithms.
        """
        return self._get_wrapped_object_().CycleNumber

    @cycle_number.setter
    def cycle_number(self, value: int):
        """
        Gets or sets the cycle number. Cycle number used to associate events within a
        scan event cycle. For example, on the first cycle of scan events, all the events
        would set this to '1'. On the second cycle, all the events would set this to
        '2'. This field must be set by devices if supporting compound names for filtering.
        However, it may be set in all acquisitions to help processing algorithms.
        """
        assert type(value) is int
        self._get_wrapped_object_().CycleNumber = value

    def clone(self) -> ScanStatistics:
        """
        Creates a new object that is a copy of the current instance.
        
        Returns:
        A new object that is a copy of this instance.
        """
        return ScanStatistics._get_wrapper_(self._get_wrapped_object().Clone())

    def copy_to(self, stats: ScanStatistics, deep: bool):
        """
        Copy all fields
        
        Parameters:
        stats:
        Copy into this object
        
        deep:
        If set, make a "deep copy" which will evaluate any lazy items and ensure no internal
        source references
        """
        self._get_wrapped_object().CopyTo(stats._get_wrapped_object_(), deep)

    def deep_clone(self) -> ScanStatistics:
        """
        Produce a deep copy of an object. Must not contain any references into the original.
        
        Returns:
        A deep clone of all objects in this
        """
        return ScanStatistics._get_wrapper_(self._get_wrapped_object().DeepClone())



from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from typing import List


class SimpleScan(NetWrapperBase):

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.SimpleScan

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    @property
    def masses(self) -> List[float]:
        """
        Gets the list of masses of each centroid
        """
        return self._get_wrapped_object_().Masses

    @property
    def intensities(self) -> List[float]:
        """
        Gets the list of Intensities for each centroid
        """
        return self._get_wrapped_object_().Intensities



from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data.business import DataUnits
from fisher_py.utils import to_net_list
from typing import List


class InstrumentData(NetWrapperBase):

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.InstrumentData

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    @property
    def name(self) -> str:
        """
        Gets or sets the name of the instrument
        """
        return self._get_wrapped_object_().Name

    @name.setter
    def name(self, value: str):
        """
        Gets or sets the name of the instrument
        """
        assert type(value) is str
        self._get_wrapped_object_().Name = value

    @property
    def model(self) -> str:
        """
        Gets or sets the model of instrument
        """
        return self._get_wrapped_object_().Model

    @model.setter
    def model(self, value: str):
        """
        Gets or sets the model of instrument
        """
        assert type(value) is str
        self._get_wrapped_object_().Model = value

    @property
    def serial_number(self) -> str:
        """
        Gets or sets the serial number of instrument
        """
        return self._get_wrapped_object_().SerialNumber

    @serial_number.setter
    def serial_number(self, value: str):
        """
        Gets or sets the serial number of instrument
        """
        assert type(value) is str
        self._get_wrapped_object_().SerialNumber = value

    @property
    def software_version(self) -> str:
        """
        Gets or sets the software version of instrument
        """
        return self._get_wrapped_object_().SoftwareVersion

    @software_version.setter
    def software_version(self, value: str):
        """
        Gets or sets the software version of instrument
        """
        assert type(value) is str
        self._get_wrapped_object_().SoftwareVersion = value

    @property
    def hardware_version(self) -> str:
        """
        Gets or sets the hardware version of instrument
        """
        return self._get_wrapped_object_().HardwareVersion

    @hardware_version.setter
    def hardware_version(self, value: str):
        """
        Gets or sets the hardware version of instrument
        """
        assert type(value) is str
        self._get_wrapped_object_().HardwareVersion = value

    @property
    def channel_labels(self) -> List[str]:
        """
        Gets or sets the Names for the channels, for UV or analog data:
        """
        return self._get_wrapped_object_().ChannelLabels

    @channel_labels.setter
    def channel_labels(self, value: List[str]):
        """
        Gets or sets the Names for the channels, for UV or analog data:
        """
        assert type(value) is list
        value = to_net_list(value, str)
        self._get_wrapped_object_().ChannelLabels = value

    @property
    def units(self) -> DataUnits:
        """
        Gets or sets the units of the Signal, for UV or analog
        """
        return DataUnits(self._get_wrapped_object_().Units)

    @units.setter
    def units(self, value: DataUnits):
        """
        Gets or sets the units of the Signal, for UV or analog
        """
        assert type(value) is DataUnits
        self._get_wrapped_object_().Units = value.value

    @property
    def flags(self) -> str:
        """
        Gets or sets additional information about this instrument.
        """
        return self._get_wrapped_object_().Flags

    @flags.setter
    def flags(self, value: str):
        """
        Gets or sets additional information about this instrument.
        """
        assert type(value) is str
        self._get_wrapped_object_().Flags = value

    @property
    def axis_label_x(self) -> str:
        """
        Gets or sets Device suggested label of X axis
        """
        return self._get_wrapped_object_().AxisLabelX

    @axis_label_x.setter
    def axis_label_x(self, value: str):
        """
        Gets or sets Device suggested label of X axis
        """
        assert type(value) is str
        self._get_wrapped_object_().AxisLabelX = value

    @property
    def axis_label_y(self) -> str:
        """
        Gets or sets Device suggested label of Y axis (name for units of data, such as
        "�C")
        """
        return self._get_wrapped_object_().AxisLabelY

    @axis_label_y.setter
    def axis_label_y(self, value: str):
        """
        Gets or sets Device suggested label of Y axis
        """
        assert type(value) is str
        self._get_wrapped_object_().AxisLabelY = value

    @property
    def is_valid(self) -> bool:
        """
        Gets or sets a value indicating whether the instrument is valid.
        """
        return self._get_wrapped_object_().IsValid

    @is_valid.setter
    def is_valid(self, value: bool):
        """
        Gets or sets a value indicating whether the instrument is valid.
        """
        assert type(value) is bool
        self._get_wrapped_object_().IsValid = value

    @property
    def has_accurate_mass_precursors(self) -> bool:
        """
        Gets a value indicating whether this file has accurate mass precursors
        """
        return self._get_wrapped_object_().HasAccurateMassPrecursors

    def clone(self) -> object:
        """
        Creates a new object that is a copy of the current instance.
        
        Returns:
        A new object that is a copy of this instance.
        """
        return self._get_wrapped_object().Clone()

    def is_tsq_quantum_file(self) -> bool:
        """
        Test if this is a TSQ quantum series file. Such files may have more accurate
        precursor mass selection.
        
        Returns:
        True if this is a raw file from a TSQ Quantum
        """
        return self._get_wrapped_object().IsTsqQuantumFile()



from fisher_py.net_wrapping import ThermoFisher
from fisher_py.data.sequence_file_writer import SequenceFileWriter


class SequenceFileWriterFactory(object):
    """
    This static factory class provides methods to create and open existing sequence
    file.
    """
    
    @staticmethod
    def create_sequence_file_writer(file_name: str, open_existing: bool) -> SequenceFileWriter:
        """
        Summary:
            Creates the sequence file writer.
        
        Parameters:
          fileName:
            Name of the file.
        
          openExisting:
            True open an existing sequence file with read/write privilege; false to create
            a new unique sequence file
        
        Returns:
            Sequence file writer object
        """
        assert type(file_name) is str
        assert type(open_existing) is bool
        net_writer = ThermoFisher.CommonCore.Data.Business.SequenceFileWriterFactory.CreateSequenceFileWriter(file_name, open_existing)
        return SequenceFileWriter._get_wrapper_(net_writer)


import enum


class PeakOptions(enum.Enum):
    """
    Features which can be set per peak, such as "reference compound"
    """
    none = 0
    Saturated = 1
    Fragmented = 2
    Merged = 4
    Exception = 8
    Reference = 16
    Modified = 32
    LockPeak = 64



from __future__ import annotations
from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data.business import Range
from fisher_py.data.business.chromatogram_signal import ChromatogramData
from fisher_py.utils import to_net_list
from typing import List


class ChromatogramSignal(NetWrapperBase):
    """
    This represents the data for a chromatogram
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.ChromatogramSignal

    def __init__(self, *args):
        """
        Initializes a new instance of the ThermoFisher.CommonCore.Data.Business.ChromatogramSignal
        class.
        
        Parameters:
        signal:
        Clone from this interface
        """
        super().__init__()

        if len(args) == 0:
            self._wrapped_object = self._wrapped_type()
        elif len(args) == 1:
            self._wrapped_object = self._wrapped_type(args[0])
        else:
            raise ValueError('Unable to create chromatogram signal')

    @property
    def signal_times(self) -> List[float]:
        """
        Gets or sets the signal times.
        
        Value:
        The signal times.
        """
        return self._get_wrapped_object_().SignalTimes

    @signal_times.setter
    def signal_times(self, value: List[float]):
        """
        Gets or sets the signal times.
        
        Value:
        The signal times.
        """
        assert type(value) is list
        value = to_net_list(value, float)
        self._get_wrapped_object_().SignalTimes = value

    @property
    def time_range(self) -> Range:
        """
        Gets the time range.
        """
        return Range._get_wrapper_(self._get_wrapped_object_().TimeRange)

    @property
    def start_time(self) -> float:
        """
        Gets the time at the start of the signal
        """
        return self._get_wrapped_object_().StartTime

    @property
    def end_time(self) -> float:
        """
        Gets the time at the end of the signal
        """
        return self._get_wrapped_object_().EndTime

    @property
    def base_peak_masses(self) -> List[float]:
        """
        Gets the signal base peak masses.
        
        Value:
        The signal base peak masses. May be null (should not be used) when HasBasePeakData
        returns false
        """
        return self._get_wrapped_object_().BasePeakMasses

    @property
    def signal_base_peak_masses(self) -> List[float]:
        """
        Gets or sets the signal base peak masses.
        
        Value:
        The signal times.
        """
        return self._get_wrapped_object_().SignalBasePeakMasses

    @signal_base_peak_masses.setter
    def signal_base_peak_masses(self, value: List[float]):
        """
        Gets or sets the signal base peak masses.
        
        Value:
        The signal times.
        """
        assert type(value) is list
        value = to_net_list(value, float)
        self._get_wrapped_object_().SignalBasePeakMasses = value

    @property
    def scans(self) -> List[int]:
        """
        Gets the signal scans.
        
        Value:
        The signal scans.
        """
        return self._get_wrapped_object_().Scans

    @property
    def signal_scans(self) -> List[int]:
        """
        Gets or sets the signal scans.
        
        Value:
        The signal scans.
        """
        return self._get_wrapped_object_().SignalScans

    @signal_scans.setter
    def signal_scans(self, value: List[int]):
        """
        Gets or sets the signal scans.
        
        Value:
        The signal scans.
        """
        assert type(value) is List[int]
        value = to_net_list(value, int)
        self._get_wrapped_object_().SignalScans = value

    @property
    def intensities(self) -> List[float]:
        """
        Gets the intensities.
        
        Value:
        The signal intensities.
        """
        return self._get_wrapped_object_().Intensities

    @property
    def signal_intensities(self) -> List[float]:
        """
        Gets or sets the signal intensities.
        
        Value:
        The signal intensities.
        """
        return self._get_wrapped_object_().SignalIntensities

    @signal_intensities.setter
    def signal_intensities(self, value: List[float]):
        """
        Gets or sets the signal intensities.
        
        Value:
        The signal intensities.
        """
        assert type(value) is list
        value = to_net_list(value, float)
        self._get_wrapped_object_().SignalIntensities = value

    @property
    def has_base_peak_data(self) -> bool:
        """
        Gets a value indicating whether there is any base peak data in this signal
        """
        return self._get_wrapped_object_().HasBasePeakData

    @property
    def length(self) -> int:
        """
        Gets the number of points in the signal
        """
        return self._get_wrapped_object_().Length

    @property
    def times(self) -> List[float]:
        """
        Gets the times.
        """
        return self._get_wrapped_object_().Times

    @staticmethod
    def from_chromatogram_data(chromatogram_data: ChromatogramData) -> List[ChromatogramSignal]:
        """
        Create an array of signals from chromatogramData. The Interface ThermoFisher.CommonCore.Data.Interfaces.IChromatogramData
        describes data read from a file (if using IRawData). This constructor converts
        to an array of type Signal, simplifying use of individual chromatograms with
        Peak integration.
        
        Parameters:
        chromatogramData:
        data (usually read from file) to convert into signals
        
        Returns:
        The constructed signals, or null if the input is null
        """
        assert type(chromatogram_data) is ChromatogramData
        signals = ChromatogramSignal._wrapped_type.FromChromatogramData(chromatogram_data._get_wrapped_object_())
        return [ChromatogramSignal._get_wrapper_(s) for s in signals]

    @staticmethod
    def from_time_and_intensity(time: List[float], intensity: List[float]) -> ChromatogramSignal:
        """
        Summary:
            Create a Chromatogram signal, from time and intensity arrays
        
        Parameters:
          time:
            array of retention times
        
          intensity:
            array of intensities at each time
        
        Returns:
            The constructed signal, or null if either of the inputs are null, or the inputs
            are not the same length
        """
        assert type(time) is list
        assert type(intensity) is list
        time = to_net_list(time, float)
        intensity = to_net_list(intensity, float)
        return ChromatogramSignal._get_wrapper_(ChromatogramSignal._wrapped_type.FromTimeAndIntensity(time, intensity))

    @staticmethod
    def from_time_intensity_scan(time: List[float], intensity: List[float], scan: List[int]) -> ChromatogramSignal:
        """
        Summary:
            Create a Chromatogram signal, from time, intensity and scan arrays
        
        Parameters:
          time:
            array of retention times
        
          intensity:
            array of intensities at each time
        
          scan:
            array of scan numbers for each time
        
        Returns:
            The constructed signal, or null if either of the inputs are null, or the inputs
            are not the same length
        """
        assert type(time) is list
        assert type(intensity) is list
        assert type(scan) is list
        time = to_net_list(time, float)
        intensity = to_net_list(intensity, float)
        scan = to_net_list(scan, int)
        return ChromatogramSignal._get_wrapper_(ChromatogramSignal._wrapped_type.FromTimeIntensityScan(time, intensity, scan))
    
    @staticmethod
    def from_time_intensity_scan_base_peak(time: List[float], intensity: List[float], scan: List[int], base_peak: List[float]) -> ChromatogramSignal:
        """
        Summary:
            Create a Chromatogram signal, from time, intensity, scan and base peak arrays
        
        Parameters:
          time:
            array of retention times
        
          intensity:
            array of intensities at each time
        
          scan:
            array of scan numbers for each time
        
          basePeak:
            Array of base peak masses for each time
        
        Returns:
            The constructed signal, or null if either of the inputs are null, or the inputs
            are not the same length
        """
        assert type(time) is list
        assert type(intensity) is list
        assert type(scan) is list
        assert type(base_peak) is list
        time = to_net_list(time, float)
        intensity = to_net_list(intensity, float)
        scan = to_net_list(scan, int)
        base_peak = to_net_list(base_peak, float)
        return ChromatogramSignal._get_wrapper_(ChromatogramSignal._wrapped_type.FromTimeIntensityScanBasePeak(time, intensity, scan, base_peak))

    @staticmethod
    def to_chromatogram_data(signals: List[ChromatogramSignal]) -> ChromatogramData:
        """
        Summary:
            Create chromatogram data interface from signals. The Interface ThermoFisher.CommonCore.Data.Interfaces.IChromatogramData
            describes data read from a file (if using IRawData).
        //
        Parameters:
          signals:
            data (usually read from file) to convert into signals
        //
        Returns:
            The constructed signals, or null if the input is null
        """
        assert type(signals) is list
        net_signals = to_net_list([s._get_wrapped_object_() for s in signals], ChromatogramSignal._wrapped_type)
        return ChromatogramData._get_wrapper_(ChromatogramSignal._wrapped_type.ToChromatogramData(net_signals))

    def clone(self) -> object:
        """
        Creates a new object that is a (deep) copy of the current instance.
        
        Returns:
        A new object that is a copy of this instance.
        """
        return self._get_wrapped_object_().Clone()

    def delay(self, delay: float):
        """
        Add a delay to all times. This is intended to support "detector delays" where
        multiple detector see the same sample at different times.
        
        Parameters:
        delay:
        The delay.
        """
        self._get_wrapped_object_().Delay(delay)

    def valid(self) -> bool:
        """
        Test if the signal is valid
        
        Returns:
        True if both times and intensities have been set, and are the same length
        """
        return self._get_wrapped_object_().Valid()




import enum


class DataUnits(enum.Enum):
    """
    Units of data from a UV or analog devices (if known).
    """
    none = 0
    AbsorbanceUnits = 1
    MilliAbsorbanceUnits = 2
    MicroAbsorbanceUnits = 3
    Volts = 4
    MilliVolts = 5
    MicroVolts = 6



import enum


class FileType(enum.Enum):
    """
    The type of the file.
    """
    NotSupported = 0
    ExperimentMethod = 1
    SampleList = 2
    ProcessingMethod = 4
    TuneMethod = 16
    ResultsFile = 32
    QuanFile = 64
    CalibrationFile = 128
    MethodFile = 256
    XqnFile = 512
    LayoutFile = 4096
    MethodEditorLayout = 4160
    SampleListEditorLayout = 4224
    ProcessingMethodEditLayout = 4352
    QualBrowserLayout = 4608
    TuneLayout = 5120
    ResultsLayout = 6144



from __future__ import annotations
from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data import ToleranceUnits
from fisher_py.utils import is_number


class MassOptions(NetWrapperBase):
    """
    Contains the options for displaying and calculating the masses.
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.MassOptions

    def __init__(self, *args):
        super().__init__()

        if len(args) == 0:
            self._wrapped_object = self._wrapped_type()
        elif len(args) == 1:
            arg = args[0]
            if type(arg) is MassOptions:
                self._wrapped_object = self._wrapped_type(arg._get_wrapped_object())
            elif is_number(arg):
                self._wrapped_object = self._wrapped_type(float(arg))
            else:
                raise ValueError('Unable to create mass options.')                
        elif len(args) == 2:
            tolerance, tolerance_units = args
            assert is_number(tolerance)
            assert type(tolerance_units) is ToleranceUnits
            self._wrapped_object = self._wrapped_type(float(tolerance), tolerance_units.value)
        elif len(args) == 3:
            tolerance, tolerance_units, precision = args
            assert is_number(tolerance)
            assert type(tolerance_units) is ToleranceUnits
            assert type(precision) is int
            self._wrapped_object = self._wrapped_type(float(tolerance), tolerance_units.value, precision)
        else:
            raise ValueError('Unable to create mass options.')

    @property
    def tolerance(self) -> float:
        """
        Gets or sets the tolerance value.
        
        Value:
        The tolerance.
        """
        return self._get_wrapped_object_().Tolerance

    @tolerance.setter
    def tolerance(self, value: float):
        """
        Gets or sets the tolerance value.
        
        Value:
        The tolerance.
        """
        assert type(value) is float
        self._get_wrapped_object_().Tolerance = value

    @property
    def precision(self) -> int:
        """
        Gets or sets the precision (decimal places).
        """
        return self._get_wrapped_object_().Precision

    @precision.setter
    def precision(self, value: int):
        """
        Gets or sets the precision (decimal places).
        """
        assert type(value) is int
        self._get_wrapped_object_().Precision = value

    @property
    def tolerance_units(self) -> ToleranceUnits:
        """
        Gets or sets the tolerance units.
        """
        return ToleranceUnits(self._get_wrapped_object_().ToleranceUnits)

    @tolerance_units.setter
    def tolerance_units(self, value: ToleranceUnits):
        """
        Gets or sets the tolerance units.
        """
        assert type(value) is ToleranceUnits
        self._get_wrapped_object_().ToleranceUnits = value.value

    @property
    def tolerance_string(self) -> str:
        """
        Gets the tolerance string of the current toleranceUnits setting.
        """
        return self._get_wrapped_object_().ToleranceString

    def get_tolerance_string(tolerance_units: ToleranceUnits) -> str:
        """
        Gets the tolerance string from the enumeration strings resource.
        
        Parameters:
        toleranceUnits:
        The tolerance units.
        
        Returns:
        The tolerance units as a string.
        """
        return MassOptions._wrapped_type.GetToleranceString(tolerance_units.value)

    def clone(self) -> MassOptions:
        """
        Implementation of ICloneable.Clone method. Creates deep copy of this instance.
        
        Returns:
        An exact copy of the current object.
        """
        return MassOptions._get_wrapper_(self._get_wrapped_object().Clone())

    def get_tolerance_at_mass(self, mass: float) -> float:
        """
        Get the tolerance window around a specific mass
        
        Parameters:
        mass:
        Mass about which window is needed
        
        Returns:
        The distance (in amu) from the mass which is within tolerance. For example: myWindow=GetToleranceAtMass(myMass);
        accept data between "myMass-myWindow" and "myMass+myWindow"
        """
        return self._get_wrapped_object().GetToleranceAtMass(mass)    



import enum


class BracketType(enum.Enum):
    """
    Specifies a sequence bracket type. This determines which groups of samples use
    the same calibration curve.
    """
    Unspecified = 0
    Overlapped = 1
    none = 2
    NonOverlapped = 3
    Open = 4



from __future__ import annotations
from typing import List
from fisher_py.utils import is_number, to_net_list
from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data.business import Range, MassOptions, SimpleScan
from fisher_py.data import PeakOptions


class SegmentedScan(NetWrapperBase):
    """
    Defines a scan with mass range segments.
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.SegmentedScan

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    @property
    def flags(self) -> List[PeakOptions]:
        """
        Gets or sets flags, such as "saturated" for each peak.
        """
        return [PeakOptions(v) for v in self._get_wrapped_object_().Flags]

    @flags.setter
    def flags(self, value: List[PeakOptions]):
        """
        Gets or sets flags, such as "saturated" for each peak.
        """
        assert type(value) is list
        value = to_net_list(value, int)
        self._get_wrapped_object_().Flags = [v.value for v in value]

    @property
    def intensities(self) -> List[float]:
        """
        Gets or sets the Intensity (or absorbance) values for each point in the scan
        """
        return self._get_wrapped_object_().Intensities

    @intensities.setter
    def intensities(self, value: List[float]):
        """
        Gets or sets the Intensity (or absorbance) values for each point in the scan
        """
        assert type(value) is list
        value = to_net_list(value, float)
        self._get_wrapped_object_().Intensities = value

    @property
    def positions(self) -> List[float]:
        """
        Gets or sets the positions (mass or wavelength) for each point in the scan
        """
        return self._get_wrapped_object_().Positions

    @positions.setter
    def positions(self, value: List[float]):
        """
        Gets or sets the positions (mass or wavelength) for each point in the scan
        """
        assert type(value) is list
        value = to_net_list(value, float)
        self._get_wrapped_object_().Positions = value

    @property
    def segment_lengths(self) -> List[int]:
        """
        Gets SegmentLengths.
        """
        return self._get_wrapped_object_().SegmentLengths

    @property
    def segment_sizes(self) -> List[int]:
        """
        Gets or sets the number of data points in each segment
        """
        return self._get_wrapped_object_().SegmentSizes

    @segment_sizes.setter
    def segment_sizes(self, value: List[int]):
        """
        Gets or sets the number of data points in each segment
        """
        assert type(value) is list
        value = to_net_list(value, int)
        self._get_wrapped_object_().SegmentSizes = value

    @property
    def segment_count(self) -> int:
        """
        Gets or sets The number of segments
        """
        return self._get_wrapped_object_().SegmentCount

    @segment_count.setter
    def segment_count(self, value: int):
        """
        Gets or sets The number of segments
        """
        assert type(value) is int
        value = to_net_list(value, int)
        self._get_wrapped_object_().SegmentCount = value

    @property
    def ranges(self) -> List[Range]:
        """
        Gets or sets the Mass ranges for each scan segment
        """
        return [Range._get_wrapper_(r) for r in self._get_wrapped_object_().Ranges]

    @ranges.setter
    def ranges(self, value: List[Range]):
        """
        Gets or sets the Mass ranges for each scan segment
        """
        assert type(value) is list
        value = to_net_list([i._get_wrapped_object_() for i in value], Range._wrapped_type)
        self._get_wrapped_object_().Ranges = value

    @property
    def mass_ranges(self) -> List[Range]:
        """
        Gets the Mass ranges for each scan segment
        """
        return [Range._get_wrapper_(r) for r in self._get_wrapped_object_().MassRanges]

    @property
    def position_count(self) -> int:
        """
        Gets or sets the The size of the position and intensity arrays. The number of
        peaks in the scan (total for all segments)
        """
        return self._get_wrapped_object_().PositionCount

    @position_count.setter
    def position_count(self, value: int):
        """
        Gets or sets the The size of the position and intensity arrays. The number of
        peaks in the scan (total for all segments)
        """
        assert type(value) is int
        value = to_net_list(value, int)
        self._get_wrapped_object_().PositionCount = value

    @property
    def scan_number(self) -> int:
        """
        Gets or sets the he number of this scan.
        """
        return self._get_wrapped_object_().ScanNumber

    @scan_number.setter
    def scan_number(self, value: int):
        """
        Gets or sets the he number of this scan.
        """
        assert type(value) is int
        value = to_net_list(value, int)
        self._get_wrapped_object_().ScanNumber = value

    def from_mass_and_intensities(self, masses: List[float], intensities: List[float]) -> SegmentedScan:
        """
        Create a scan from simple X,Y data. This method creates a scan with one segment.
        For efficiency, the references to the mass and intensity data are maintained
        within the constructed object. If this is not desired, clone the mass and intensity
        arrays on calling this constructor. Masses are assumed to be in ascending order.
        
        Parameters:
        masses:
        Mass data for the scan
        
        intensities:
        Intensity data for the scan
        
        Returns:
        A scan with one segment
        
        Exceptions:
        T:System.ArgumentNullException:
        masses is null.
        
        T:System.ArgumentNullException:
        intensities is null.
        
        T:System.ArgumentException:
        Intensities must have same length as masses
        """
        return SegmentedScan._get_wrapper_(self._get_wrapped_object().FromMassesAndIntensities(masses, intensities))

    def base_intensity(self, ranges: List[Range], tolerance_options: MassOptions) -> float:
        """
        Return the largest intensity (base value) in the ranges supplied
        
        Parameters:
        ranges:
        Ranges of positions (masses, wavelengths)
        
        toleranceOptions:
        If the ranges have equal mass values, then toleranceOptions are used to determine
        a band subtracted from low and added to high to search for matching masses
        
        Returns:
        Largest intensity in all ranges
        """
        assert type(ranges) is list
        assert type(tolerance_options) is MassOptions
        net_ranges = to_net_list([r._get_wrapped_object() for r in ranges], Range._wrapped_type)
        return self._get_wrapped_object().BaseIntensity(net_ranges, tolerance_options._get_wrapped_object_())

    def base_intensity(self, ranges: List[Range], tolerance: float) -> float:
        """
        Return the largest intensity (base value) in the ranges supplies
        
        Parameters:
        ranges:
        Ranges of positions
        
        tolerance:
        If the ranges have equal mass values, then tolerance is subtracted from low and
        added to high to search for matching masses
        
        Returns:
        The largest intensity in all ranges
        """
        assert type(ranges) is list
        assert is_number(tolerance)
        net_ranges = to_net_list([r._get_wrapped_object() for r in ranges], Range._wrapped_type)
        return self._get_wrapped_object().BaseIntensity(net_ranges, float(tolerance))

    def clone(self) -> SegmentedScan:
        """
        Creates a new object that is a copy of the current instance.
        
        Returns:
        A new object that is a copy of this instance.
        """
        return SegmentedScan._get_wrapper_(self._get_wrapped_object().Clone())

    def deep_clone(self) -> SegmentedScan:
        """
        Make a deep clone of this object.
        
        Returns:
        An object containing all data in this, and no shared references
        """
        return SegmentedScan._get_wrapper_(self._get_wrapped_object().DeepClone())

    def index_of_segment_start(self, segment: int) -> int:
        """
        Find the index of the first packet in a segment
        
        Parameters:
        segment:
        segment number (starting from 0)
        
        Returns:
        the index of the first packet in a segment
        """
        return self._get_wrapped_object().IndexOfSegmentStart(segment)

    def sum_intensities(self, ranges: List[Range], tolerance_options: MassOptions) -> float:
        """
        Sum all masses within the ranges
        
        Parameters:
        ranges:
        List of ranges to sum
        
        toleranceOptions:
        If the ranges have equal mass values, then toleranceOptions are used to determine
        a band subtracted from low and added to high to search for matching masses
        
        Returns:
        Sum of intensities in all ranges
        """
        assert type(ranges) is list
        assert type(tolerance_options) is MassOptions
        net_ranges = to_net_list([r._get_wrapped_object() for r in ranges], Range._wrapped_type)
        return self._get_wrapped_object().SumIntensities(net_ranges, tolerance_options._get_wrapped_object_())

    def sum_intensities(self, ranges: List[Range], tolerance: float) -> float:
        """
        Sum all masses within the ranges
        
        Parameters:
        ranges:
        List of ranges to sum
        
        tolerance:
        If the ranges have equal mass values, then tolerance is subtracted from low and
        added to high to search for matching masses
        
        Returns:
        Sum of intensities in all ranges
        """
        return self._get_wrapped_object().SumIntensities(ranges, tolerance)

    def to_simple_scan(self) -> SimpleScan:
        """
        Convert to simple scan. This permits calling code to free up references to unused
        parts of the scan data.
        
        Returns:
        The ThermoFisher.CommonCore.Data.Interfaces.ISimpleScanAccess.
        """
        return SimpleScan._get_wrapper_(self._get_wrapped_object().ToSimpleScan())

    def try_validate(self) -> bool:
        """
        Test if this is a valid object (all streams are not null. All data has same length)
        
        Returns:
        True if valid.
        """
        return self._get_wrapped_object().TryValidate()

    def validate(self):
        """
        Test if this is a valid object (all streams are not null. All data has same length)
        
        Exceptions:
        T:System.ArgumentException:
        is thrown if this instance does not contain valid data.
        """
        self._get_wrapped_object().Validate()



import enum

class SampleType(enum.Enum):
    """
    Enumeration of sample types
    """
    Unknown = 0
    Blank = 1
    QC = 2
    StdClear = 3
    StdUpdate = 4
    StdBracket = 5
    StdBracketStart = 6
    StdBracketEnd = 7
    Program = 8
    SolventBlank = 9
    MatrixBlank = 10
    MatrixSpike = 11
    MatrixSpikeDuplicate = 12



import enum

class ToleranceMode(enum.Enum):
    """
    Specifies units for measuring mass tolerance.
    Tolerance is used to determine if a results should be kept, in formula search.
    If the exact mass of a formula is not within tolerance of a measured mass from
    an instrument, then the formula is not considered a valid result.
    """
    none = 0,
    Amu = 1,
    Mmu = 2,
    Ppm = 3



from fisher_py.net_wrapping import NetWrapperBase
from typing import List


class ScanDependentDetails(NetWrapperBase):

    @property
    def scan_index(self) -> int:
        """
        Gets the index of the scan.
        
        Value:
        The index of the scan.
        """
        return self._get_wrapped_object_().ScanIndex

    @property
    def filter_string(self) -> str:
        """
        Gets the filter string.
        
        Value:
        The filter string.
        """
        return self._get_wrapped_object_().FilterString

    @property
    def precursor_mass_array(self) -> List[float]:
        """
        Gets the precursor array.
        
        Value:
        The precursor mass array.
        """
        return self._get_wrapped_object_().PrecursorMassArray

    @property
    def isolation_width_array(self) -> List[float]:
        """
        Gets the isolation width array.
        
        Value:
        The isolation width array.
        """
        return self._get_wrapped_object_().IsolationWidthArray




import clr
import os
from fisher_py.net_wrapping.net_wrapper_base import NetWrapperBase


# access .net standard dlls
dll_path = os.path.join(os.path.split(__file__)[0], '..', 'dll')
clr.AddReference('System')
clr.AddReference('System.Core')
clr.AddReference('System.Data')
clr.AddReference('System.Configuration')
clr.AddReference('System.Xml')
clr.AddReference('mscorlib')

clr.AddReference(os.path.join(dll_path, 'ThermoFisher.CommonCore.Data.dll'))
clr.AddReference(os.path.join(dll_path, 'ThermoFisher.CommonCore.RawFileReader.dll'))
clr.AddReference(os.path.join(dll_path, 'ThermoFisher.CommonCore.MassPrecisionEstimator.dll'))
clr.AddReference(os.path.join(dll_path, 'ThermoFisher.CommonCore.BackgroundSubtraction.dll'))
clr.AddReference(os.path.join(dll_path, 'OpenMcdf.dll'))

# import .net standard libaries
import ThermoFisher.CommonCore.Data as thermo_fisher_data
from ThermoFisher.CommonCore.Data import Extensions
import ThermoFisher.CommonCore.Data.Business as thermo_fisher_data_business
import ThermoFisher.CommonCore.Data.FilterEnums as thermo_fisher_data_filter_enums
import ThermoFisher.CommonCore.Data.Interfaces as thermo_fisher_data_interfaces
import ThermoFisher.CommonCore.MassPrecisionEstimator as thermo_fisher_mass_precision_estimator
import ThermoFisher.CommonCore.RawFileReader as thermo_fisher_raw_file_reader
import ThermoFisher.CommonCore.MassPrecisionEstimator as thermo_fisher_mass_precision_estimator

# expose .net standard libraries in a friendly manner
class _Data:

    Business = thermo_fisher_data_business
    FilterEnums = thermo_fisher_data_filter_enums
    Interfaces = thermo_fisher_data_interfaces
    Extensions = Extensions


class ThermoFisher:

    class CommonCore:
        Data = _Data()
        MassPrecisionEstimator = thermo_fisher_mass_precision_estimator
        RawFileReader = thermo_fisher_raw_file_reader
        MassPrecisionEstimator = thermo_fisher_mass_precision_estimator



import enum

class SpectrumPacketType(enum.Enum):
    """
    The spectrum packet types. Internally, within raw files, these are defined simply
    as "a short integer packet type" These are then mapped to "constants". It is
    possible that types may be returned by from raw data, or other transmissions,
    which are outside of this range. These types define original compression formats
    from instruments. Note that most data values are returned as "float", when using
    IRawDataPlus regardless of the compressed file format used.
    """
    NoPacketType = -1
    ProfileSpectrum = 0
    LowResolutionSpectrum = 1
    HighResolutionSpectrum = 2
    ProfileIndex = 3
    CompressedAccurateSpectrum = 4
    StandardAccurateSpectrum = 5
    StandardUncalibratedSpectrum = 6
    AccurateMassProfileSpectrum = 7
    PdaUvDiscreteChannel = 8
    PdaUvDiscreteChannelIndex = 9
    PdaUvScannedSpectrum = 10
    PdaUvScannedSpectrumIndex = 11
    UvChannel = 12
    MassSpecAnalog = 13
    ProfileSpectrumType2 = 14
    LowResolutionSpectrumType2 = 15
    ProfileSpectrumType3 = 16
    LowResolutionSpectrumType3 = 17
    LinearTrapCentroid = 18
    LinearTrapProfile = 19,
    FtCentroid = 20
    FtProfile = 21
    HighResolutionCompressedProfile = 22
    LowResolutionCompressedProfile = 23
    LowResolutionSpectrumType4 = 24
    InvalidPacket = 25




from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from typing import List


class LogEntry(NetWrapperBase):
    """
    Represents a single log.
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.LogEntry

    @property
    def labels(self) -> List[str]:
        """
        Gets or sets the labels in this log.
        """
        return list(self._get_wrapped_object_().Labels)

    @labels.setter
    def labels(self, value: List[str]):
        """
        Gets or sets the labels in this log.
        """
        assert type(value) is List[str]
        self._get_wrapped_object_().Labels = value

    @property
    def values(self) -> List[str]:
        """
        Gets or sets the values in this log.
        """
        return list(self._get_wrapped_object_().Values)

    @values.setter
    def values(self, value: List[str]):
        """
        Gets or sets the values in this log.
        """
        assert type(value) is List[str]
        self._get_wrapped_object_().Values = value

    @property
    def length(self) -> int:
        """
        Gets or sets the length of the log.
        """
        return self._get_wrapped_object_().Length

    @length.setter
    def length(self, value: int):
        """
        Gets or sets the length of the log.
        """
        assert type(value) is int
        self._get_wrapped_object_().Length = value




from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.utils import is_number


class NoiseAndBaseline(NetWrapperBase):
    """
    Defines noise and baseline at a given mass (Part of support for reading orbitrap data)
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.NoiseAndBaseline

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    @property
    def mass(self) -> float:
        """
        Gets or sets the mass.
        """
        return self._get_wrapped_object_().Mass

    @mass.setter
    def mass(self, value: float):
        """
        Gets or sets the mass.
        """
        assert is_number(value)
        self._get_wrapped_object_().Mass = float(value)

    @property
    def noise(self) -> float:
        """
        Gets or sets the noise.
        """
        return self._get_wrapped_object_().Noise

    @noise.setter
    def noise(self, value: float):
        """
        Gets or sets the noise.
        """
        assert is_number(value)
        self._get_wrapped_object_().Noise = float(value)

    @property
    def baseline(self) -> float:
        """
        Gets or sets the baseline.
        """
        return self._get_wrapped_object_().Baseline

    @baseline.setter
    def baseline(self, value: float):
        """
        Gets or sets the baseline.
        """
        assert is_number(value)
        self._get_wrapped_object_().Baseline = float(value)




from __future__ import annotations
from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data import PeakOptions
from fisher_py.data.business import Range, LabelPeak, MassOptions, SimpleScan, SegmentedScan, ScanStatistics
from fisher_py.utils import to_net_list
from typing import List, TYPE_CHECKING

if TYPE_CHECKING:
    from fisher_py.data.business import Scan


class CentroidStream(NetWrapperBase):

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.CentroidStream

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    @property
    def noises(self) -> List[float]:
        """
        Gets or sets the list of noise level near peak
        """
        return list(self._get_wrapped_object_().Noises)

    @noises.setter
    def noises(self, value: List[float]):
        """
        Gets or sets the list of noise level near peak
        """
        assert type(value) is list
        value = to_net_list(value, float)
        self._get_wrapped_object_().Noises = value

    @property
    def masses(self) -> List[float]:
        """
        Gets or sets the list of masses of each centroid
        """
        return list(self._get_wrapped_object_().Masses)

    @masses.setter
    def masses(self, value: List[float]):
        """
        Gets or sets the list of masses of each centroid
        """
        assert type(value) is list
        value = to_net_list(value, float)
        self._get_wrapped_object_().Masses = value

    @property
    def length(self) -> int:
        """
        Gets or sets the number of centroids
        """
        return self._get_wrapped_object_().Length

    @length.setter
    def length(self, value: int):
        """
        Gets or sets the number of centroids
        """
        assert type(value) is int
        self._get_wrapped_object_().Length = value

    @property
    def intensities(self) -> List[float]:
        """
        Gets or sets the list of Intensities for each centroid
        """
        return list(self._get_wrapped_object_().Intensities)

    @intensities.setter
    def intensities(self, value: List[float]):
        """
        Gets or sets the list of Intensities for each centroid
        """
        assert type(value) is list
        value = to_net_list(value, float)
        self._get_wrapped_object_().Intensities = value

    @property
    def flags(self) -> List[PeakOptions]:
        """
        Gets or sets the flags for the peaks (such as reference)
        """
        return [PeakOptions(f) for f in self._get_wrapped_object_().Flags]

    @flags.setter
    def flags(self, value: List[PeakOptions]):
        """
        Gets or sets the flags for the peaks (such as reference)
        """
        assert type(value) is list
        value = to_net_list([v.value for v in value], int)
        self._get_wrapped_object_().Flags = value

    @property
    def coefficients_count(self) -> int:
        """
        Gets or sets the coefficients count.
        """
        return self._get_wrapped_object_().CoefficientsCount

    @coefficients_count.setter
    def coefficients_count(self, value: int):
        """
        Gets or sets the coefficients count.
        """
        assert type(value) is int
        self._get_wrapped_object_().CoefficientsCount = value

    @property
    def coefficients(self) -> List[float]:
        """
        Gets or sets the calibration Coefficients
        """
        return list(self._get_wrapped_object_().Coefficients)

    @coefficients.setter
    def coefficients(self, value: List[float]):
        """
        Gets or sets the calibration Coefficients
        """
        assert type(value) is list
        self._get_wrapped_object_().Coefficients = value

    @property
    def charges(self) -> List[float]:
        """
        Gets or sets the list of charge calculated for peak
        """
        return list(self._get_wrapped_object_().Charges)

    @charges.setter
    def charges(self, value: List[float]):
        """
        Gets or sets the list of charge calculated for peak
        """
        assert type(value) is list
        value = to_net_list(value, float)
        self._get_wrapped_object_().Charges = value

    @property
    def base_peak_resolution(self) -> float:
        """
        Gets the resolution of most intense peak
        """
        return self._get_wrapped_object_().BasePeakResolution

    @property
    def base_peak_noise(self) -> float:
        """
        Gets the noise of most intense peak
        """
        return self._get_wrapped_object_().BasePeakNoise

    @property
    def base_peak_mass(self) -> float:
        """
        Gets the mass of most intense peak
        """
        return self._get_wrapped_object_().BasePeakMass

    @property
    def base_peak_intensity(self) -> float:
        """
        Gets the intensity of most intense peak
        """
        return self._get_wrapped_object_().BasePeakIntensity

    @property
    def baselines(self) -> List[float]:
        """
        Gets or sets the list of baseline at each peak
        """
        return list(self._get_wrapped_object_().Baselines)

    @baselines.setter
    def baselines(self, value: List[float]):
        """
        Gets or sets the list of baseline at each peak
        """
        assert type(value) is list
        value = to_net_list(value, float)
        self._get_wrapped_object_().Baselines = value

    @property
    def resolutions(self) -> List[float]:
        """
        Gets or sets resolution of each peak
        """
        return list(self._get_wrapped_object_().Resolutions)

    @resolutions.setter
    def resolutions(self, value: List[float]):
        """
        Gets or sets resolution of each peak
        """
        assert type(value) is list
        value = to_net_list(value, float)
        self._get_wrapped_object_().Resolutions = value

    @property
    def scan_number(self) -> int:
        """
        Gets or sets the scan Number
        """
        return self._get_wrapped_object_().ScanNumber

    @scan_number.setter
    def scan_number(self, value: int):
        """
        Gets or sets the scan Number
        """
        assert type(value) is int
        self._get_wrapped_object_().ScanNumber = value

    def base_intensity(self, ranges: List[Range], tolerance_options: MassOptions) -> float:
        """
        Return the largest intensity (base value) in the ranges supplies
        
        Parameters:
        ranges:
        Ranges of masses
        
        toleranceOptions:
        If the ranges have equal mass values, then toleranceOptions are used to determine
        a band subtracted from low and added to high to search for matching masses
        
        Returns:
        Largest intensity in all ranges
        """
        net_ranges = [v.value for v in ranges]
        return self._get_wrapped_object().BaseIntensity(net_ranges, tolerance_options._get_wrapped_object_())

    def clear(self):
        """
        Clears all the data.
        """
        self._get_wrapped_object().Clear()

    def clone(self) -> object:
        """
        Creates a new object that is a copy of the current instance.
        
        Returns:
        A new object that is a copy of this instance.
        """
        return self._get_wrapped_object().Clone()

    def deep_clone(self) -> CentroidStream:
        """
        Make a deep clone of this object.
        
        Returns:
        An object containing all data in this, and no shared references
        """
        return CentroidStream._get_wrapper_(self._get_wrapped_object().DeepClone())

    def get_centroids(self) -> List[LabelPeak]:
        """
        Get the list centroids.
        
        Returns:
        The centroids in the scan
        """
        return [LabelPeak._get_wrapper_(c) for c in self._get_wrapped_object().GetCentroids()]

    def get_label_peak(self, index: int) -> LabelPeak:
        """
        Convert the data for a given peak in this stream into a LabelPeak
        
        Parameters:
        index:
        The index of the peak to convert.
        
        Returns:
        Extracted data for the selected peak
        """
        return LabelPeak._get_wrapper_(self._get_wrapped_object().GetLabelPeak(index))

    def get_label_peaks(self) -> List[LabelPeak]:
        """
        Convert the data into LabelPeak objects
        
        Returns:
        """
        return [LabelPeak._get_wrapper_(p) for p in self._get_wrapped_object().GetLabelPeaks()]

    def refresh_base_details(self):
        """
        Forces re-computation of Base peaks , intensities.
        """
        self._get_wrapped_object().RefreshBaseDetails()

    def set_label_peaks(self, label_peaks: List[LabelPeak]) -> bool:
        """
        Convert data into this object from an array of LabelPeaks
        
        Parameters:
        labelPeaks:
        
        Returns:
        true on success. False if the labels peaks are null or empty
        """
        return self._get_wrapped_object().SetLabelPeaks([p._wrapped_object for p in label_peaks])

    def sum_intensities(self, ranges: List[Range], tolerance_options: MassOptions) -> float:
        """
        Sum all masses within the ranges
        
        Parameters:
        ranges:
        List of ranges to sum
        
        toleranceOptions:
        If the ranges have equal mass values, then toleranceOptions are used to determine
        a band subtracted from low and added to high to search for matching masses
        
        Returns:
        Sum of intensities in all ranges
        """
        net_ranges = to_net_list([v.value for v in ranges], Range._wrapped_type)
        return self._get_wrapped_object().SumIntensities(net_ranges, tolerance_options._get_wrapped_object_())

    def sum_masses(self, ranges: List[Range], tolerance: float) -> float:
        """
        Sum all masses within the ranges
        
        Parameters:
        ranges:
        List of ranges to sum
        
        tolerance:
        If the ranges have equal mass values, then tolerance is subtracted from low and
        added to high to search for matching masses
        
        Returns:
        Sum of intensities in all ranges
        """
        net_ranges = to_net_list([v.value for v in ranges], Range._wrapped_type)
        return self._get_wrapped_object().SumMasses(net_ranges, tolerance)

    def to_scan(self, original_scan_stats: ScanStatistics) -> Scan:
        """
        Convert to Scan. This feature is intended for use where an application or algorithm
        needs data in "Scan" format, with centroid information in the "SegmentedScan"
        property of the Scan. (such as typical centroid data from ITMS), but the data
        in this scan came from an FTMS detector, which would have the profile data in
        "SegmentedScan" and the centroid data in this object. The data is first converted
        to SegmentedScan format (using ToSegmentedScan) then a new Scan is made containing
        that data (with no data in "CentroidStream). Data from this object is duplicated
        (deep copy), such that changing values in the returned object will not affect
        data in this object. This initializes the returned scan's "ScanStatistics" based
        on the returned mass and intensity data. If the (optional) originalScanStats
        parameter is included, information from that is used to initialize RT, scan number
        and other fields which cannot be calculated from this data. The only values updated
        in the scan statistics are "BasePeakMass" and "BasePeakIntenity". All other values
        are either as copied from the supplied parameter, or defaults. Application should
        set any other values needed in the Scan, such as "ScansCombined, ToleranceUnit,
        MassResolution", which cannot be determined from the supplied parameters.
        
        Parameters:
        originalScanStats:
        If this is supplied, the scan statistics are initialized as a deep clone of the
        supplied object (so that RT etc. get preserved) then the values of BasePeakMass
        and BasePeakIntensity are updated from this object
        
        Returns:
        The ThermoFisher.CommonCore.Data.Business.SegmentedScan.
        """
        from fisher_py.data.business import Scan
        return Scan._get_wrapper_(self._get_wrapped_object().ToScan(original_scan_stats._get_wrapped_object_()))

    def to_segmented_scan(self) -> SegmentedScan:
        """
        Convert to segmented scan. This feature is intended for use where an application
        or algorithm in "SegmentedScan" format, such as typical centroid data from ITMS,
        but the data in this scan came from an FTMS detector, which would have the profile
        data in "SegmentedScan" and the centroid data in this object. Data from this
        object is duplicated (deep copy), such that changing values in the returned object
        will not affect data in this object.
        
        Returns:
        The ThermoFisher.CommonCore.Data.Business.SegmentedScan.
        """
        return SegmentedScan._get_wrapper_(self._get_wrapped_object().ToSegmentedScan())

    def to_simple_scan(self) -> SimpleScan:
        """
        there can be an advantage in doing this conversion, as when this object goes
        out of scope the converted object only holds the mass and intensity refs, and
        will need less memory.
        
        Returns:
        The ThermoFisher.CommonCore.Data.Business.SimpleScan.
        """
        return SimpleScan._get_wrapper_(self._get_wrapped_object().ToSimpleScan())

    def try_validate(self) -> bool:
        """
        Test if this is a valid object (all streams are not null. All data has same length)
        
        Returns:
        true if the object has valid data in it.
        """
        return self._get_wrapped_object().TryValidate()

    def validate(self):
        """
        Test if this is a valid object (all streams are not null. All data has same length)
        
        Exceptions:
        T:System.ArgumentException:
        is thrown if this instance does not contain valid data.
        """
        self._get_wrapped_object().Validate()



from __future__ import annotations
from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from fisher_py.raw_file_reader import RawFileAccess
    from fisher_py.data.business import Scan


class CachedScanProvider(NetWrapperBase):

    def __init__(self):
        super().__init__()

    def get_scan_at_time(self, raw_data: RawFileAccess, time: float) -> Scan:
        """
        Create a scan object from a file and a retention time.
        
        Parameters:
        rawData:
        File to read from
        
        time:
        time of Scan number to read
        
        Returns:
        The scan read, or null if no scan was read
        """
        from fisher_py.data.business import Scan
        return Scan._get_wrapper_(self._get_wrapped_object().GetScanAtTime(raw_data._get_wrapped_object_(), time))

    def get_scan_from_scan_number(self, raw_data: RawFileAccess, scan_number: int) -> Scan:
        """
        Create a scan object from a file and a scan number.
        
        Parameters:
        rawData:
        File to read from
        
        scanNumber:
        Scan number to read
        
        Returns:
        The scan read, or null of the scan number if not valid
        """
        from fisher_py.data.business import Scan
        return Scan._get_wrapper_(self._get_wrapped_object().GetScanFromScanNumber(raw_data._get_wrapped_object_(), scan_number))



from __future__ import annotations
from typing import List
from fisher_py.net_wrapping import ThermoFisher
from fisher_py.data import CommonCoreDataObject
from fisher_py.data.business import SampleType, BarcodeStatusType
from fisher_py.utils import to_net_list


class SampleInformation(CommonCoreDataObject):
    """
    Encapsulates various information about sample.
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.SampleInformation

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    max_user_text_column_count = 20

    @property
    def sample_volume(self) -> float:
        """
        Gets or sets the sample volume of this sequence row.
        """
        return self._get_wrapped_object_().SampleVolume

    @sample_volume.setter
    def sample_volume(self, value: float):
        """
        Gets or sets the sample volume of this sequence row.
        """
        assert type(value) is float
        self._get_wrapped_object_().SampleVolume = value

    @property
    def sample_type(self) -> SampleType:
        """
        Gets or sets the type of the sample.
        """
        return SampleType(self._get_wrapped_object_().SampleType)

    @sample_type.setter
    def sample_type(self, value: SampleType):
        """
        Gets or sets the type of the sample.
        """
        assert type(value) is SampleType
        self._get_wrapped_object_().SampleType = value.value

    @property
    def processing_method_file(self) -> str:
        """
        Gets or sets the processing method filename of this sequence row.
        """
        return self._get_wrapped_object_().ProcessingMethodFile

    @processing_method_file.setter
    def processing_method_file(self, value: str):
        """
        Gets or sets the processing method filename of this sequence row.
        """
        assert type(value) is str
        self._get_wrapped_object_().ProcessingMethodFile = value

    @property
    def path(self) -> str:
        """
        Gets or sets the path to original data.
        """
        return self._get_wrapped_object_().Path

    @path.setter
    def path(self, value: str):
        """
        Gets or sets the path to original data.
        """
        assert type(value) is str
        self._get_wrapped_object_().Path = value

    @property
    def row_number(self) -> int:
        """
        Gets or sets the row number.
        """
        return self._get_wrapped_object_().RowNumber

    @row_number.setter
    def row_number(self, value: int):
        """
        Gets or sets the row number.
        """
        assert type(value) is int
        self._get_wrapped_object_().RowNumber = value

    @property
    def istd_amount(self) -> float:
        """
        Gets or sets the ISTD amount of this sequence row.
        """
        return self._get_wrapped_object_().IstdAmount

    @istd_amount.setter
    def istd_amount(self, value: float):
        """
        Gets or sets the ISTD amount of this sequence row.
        """
        assert type(value) is float
        self._get_wrapped_object_().IstdAmount = value

    @property
    def calibration_file(self) -> str:
        """
        Gets or sets the name of calibration file.
        """
        return self._get_wrapped_object_().CalibrationFile

    @calibration_file.setter
    def calibration_file(self, value: str):
        """
        Gets or sets the name of calibration file.
        """
        assert type(value) is str
        self._get_wrapped_object_().CalibrationFile = value

    @property
    def raw_file_name(self) -> str:
        """
        Gets or sets the name of acquired file (excluding path).
        """
        return self._get_wrapped_object_().RawFileName

    @raw_file_name.setter
    def raw_file_name(self, value: str):
        """
        Gets or sets the name of acquired file (excluding path).
        """
        assert type(value) is str
        self._get_wrapped_object_().RawFileName = value

    @property
    def instrument_method_file(self) -> str:
        """
        Gets or sets the instrument method filename of this sequence row.
        """
        return self._get_wrapped_object_().InstrumentMethodFile

    @instrument_method_file.setter
    def instrument_method_file(self, value: str):
        """
        Gets or sets the instrument method filename of this sequence row.
        """
        assert type(value) is str
        self._get_wrapped_object_().InstrumentMethodFile = value

    @property
    def dilution_factor(self) -> float:
        """
        Gets or sets the bulk dilution factor (volume correction) of this sequence row.
        """
        return self._get_wrapped_object_().DilutionFactor

    @dilution_factor.setter
    def dilution_factor(self, value: float):
        """
        Gets or sets the bulk dilution factor (volume correction) of this sequence row.
        """
        assert type(value) is float
        self._get_wrapped_object_().DilutionFactor = value

    @property
    def calibration_level(self) -> str:
        """
        Gets or sets a name to identify the Calibration or QC level associated with this
        sample. Empty if this sample does not contain any calibration compound.
        """
        return self._get_wrapped_object_().CalibrationLevel

    @calibration_level.setter
    def calibration_level(self, value: str):
        """
        Gets or sets a name to identify the Calibration or QC level associated with this
        sample. Empty if this sample does not contain any calibration compound.
        """
        assert type(value) is str
        self._get_wrapped_object_().CalibrationLevel = value

    @property
    def barcode_status(self) -> BarcodeStatusType:
        """
        Gets or sets the bar code status.
        """
        return BarcodeStatusType(self._get_wrapped_object_().BarcodeStatus)

    @barcode_status.setter
    def barcode_status(self, value: BarcodeStatusType):
        """
        Gets or sets the bar code status.
        """
        assert type(value) is BarcodeStatusType
        self._get_wrapped_object_().BarcodeStatus = value.value

    @property
    def barcode(self) -> str:
        """
        Gets or sets bar code from scanner (if attached).
        """
        return self._get_wrapped_object_().Barcode

    @barcode.setter
    def barcode(self, value: str):
        """
        Gets or sets bar code from scanner (if attached).
        """
        assert type(value) is str
        self._get_wrapped_object_().Barcode = value

    @property
    def injection_volume(self) -> float:
        """
        Gets or sets the amount of sample injected.
        """
        return self._get_wrapped_object_().InjectionVolume

    @injection_volume.setter
    def injection_volume(self, value: float):
        """
        Gets or sets the amount of sample injected.
        """
        assert type(value) is float
        self._get_wrapped_object_().InjectionVolume = value

    @property
    def vial(self) -> str:
        """
        Gets or sets the vial or well form auto sampler.
        """
        return self._get_wrapped_object_().Vial

    @vial.setter
    def vial(self, value: str):
        """
        Gets or sets the vial or well form auto sampler.
        """
        assert type(value) is str
        self._get_wrapped_object_().Vial = value

    @property
    def sample_name(self) -> str:
        """
        Gets or sets the description of sample.
        """
        return self._get_wrapped_object_().SampleName

    @sample_name.setter
    def sample_name(self, value: str):
        """
        Gets or sets the description of sample.
        """
        assert type(value) is str
        self._get_wrapped_object_().SampleName = value

    @property
    def sample_id(self) -> str:
        """
        Gets or sets the Code to identify sample.
        """
        return self._get_wrapped_object_().SampleId

    @sample_id.setter
    def sample_id(self, value: str):
        """
        Gets or sets the Code to identify sample.
        """
        assert type(value) is str
        self._get_wrapped_object_().SampleId = value

    @property
    def comment(self) -> str:
        """
        Gets or sets the comment about sample (from user).
        """
        return self._get_wrapped_object_().Comment

    @comment.setter
    def comment(self, value: str):
        """
        Gets or sets the comment about sample (from user).
        """
        assert type(value) is str
        self._get_wrapped_object_().Comment = value

    @property
    def sample_weight(self) -> float:
        """
        Gets or sets the sample weight of this sequence row.
        """
        return self._get_wrapped_object_().SampleWeight

    @sample_weight.setter
    def sample_weight(self, value: float):
        """
        Gets or sets the sample weight of this sequence row.
        """
        assert type(value) is float
        self._get_wrapped_object_().SampleWeight = value

    @property
    def user_text(self) -> List[str]:
        """
        Gets or sets the collection of user text.
        """
        return self._get_wrapped_object_().UserText

    @user_text.setter
    def user_text(self, value: List[str]):
        """
        Gets or sets the collection of user text.
        """
        assert type(value) is list
        value = to_net_list(value, str)
        self._get_wrapped_object_().UserText = value

    def deep_copy(self) -> SampleInformation:
        """
        Create a deep copy of the current object.
        
        Returns:
        A deep copy of the current object.
        """
        return SampleInformation._get_wrapper_(self._get_wrapped_object().DeepCopy())



"""
A Python example program showing how to use RAWFileReader.  More information on the RAWFileReader methods used
in this example and the other methods available in RAWFileReader can be found in the RAWFileReader user
documentation, that is installed with the RAWFileReader software.
This program has been tested with RAWFileReader 4.0.22  Changes maybe necessary with other versions
of RAWFileReader.
"""

import sys
import os
import traceback
from typing import List
from fisher_py.raw_file_reader import RawFileReaderAdapter, RawFileAccess
from fisher_py.data.business import GenericDataTypes, ChromatogramTraceSettings, TraceType, ChromatogramSignal, SpectrumPacketType, Scan
from fisher_py.data.filter_enums import MsOrderType
from fisher_py.data import Device, ToleranceUnits
from fisher_py.mass_precision_estimator import PrecisionEstimate


class InclusionListItem(object):
    """
    The object used to store the inclusion/exclusion list items.
    """
    def __init__(self, descriptor: str, mass: float, threshold: float):
        self.descriptor = descriptor
        self.mass = mass
        self.threshold = threshold
        self.scan_number = 0


def list_trailer_extra_fields(raw_file: RawFileAccess):
    """
    Reads and reports the trailer extra data fields present in the RAW file.
    """
    # Get the Trailer Extra data fields present in the RAW file
    trailer_fields = raw_file.get_trailer_extra_header_information()

    # Display each value
    i = 0
    print('Trailer Extra Data Information:')

    for field in trailer_fields:
        print(f'   Field {i} = {field.label} storing data of type {field.data_type}')
        i += 1
    print('')


def list_status_log(raw_file: RawFileAccess, start_scan: int, end_scan: int):
    """
    Reads and reports the status log data fields present in the RAW file.
    """
    # Get the status log header information
    status_log = raw_file.get_status_log_header_information()

    # Display each value that is part of the status log.  They are stored as label/data type pairs
    i = 0
    print('Status Log Information:')

    for field in status_log:
        if field.label is not None and field != '' and field.data_type != GenericDataTypes.NULL:
            print(f'   Field {i} = {field.label} storing data of type {field.data_type}')
        i += 1
    print('')

    # Display the value for item 10 in the status log for each scan
    print('Status Information for item 10:')

    for scan in range(start_scan, end_scan):
        # Get the status log for this scan
        time = raw_file.retention_time_from_scan_number(scan)
        log_entry = raw_file.get_status_log_for_retention_time(time)

        # Print the values for one item
        print(f'   Scan {scan} = {log_entry.values[10]}')
    print('')


def get_inclusion_exclusion_list(raw_file: RawFileAccess, mass_tolerance: float) -> List[InclusionListItem]:
    """
    Reads the inclusion/exclusion list from the mass spectrometer method in the RAW file
    """
    # Select the MS instrument
    raw_file.select_instrument(Device.MS, 1)

    # Get the instrument method item(s) and look for the inclusion/exclusion list
    # which will be flagged by the "Mass List Table"
    inclusion_strings = list()

    for i in range(raw_file.instrument_methods_count):
        method_text = raw_file.get_instrument_method(i)
        if method_text is not None and 'Mass List Table' in method_text:
            save_line = False
            split_method = method_text.split('\n')

            for line in split_method:
                if 'Mass List Table' in line:
                    save_line = True
                elif 'End Mass List Table' in line:
                    save_line = False
                elif save_line:
                    inclusion_strings.append(line)
    
    # Create the inclusion/exclusion list 
    inclusion_list = list()

    # Convert each line from the inclusion/exclusion mass table into InclusionListItem objects
    # and add them to the inclusion/exclusion list.
    for line in inclusion_strings:
        # Skip the title line
        if 'CompoundName' in line:
            continue

        #  Split the line into its separate fields
        fields = line.split('|')

        if len(fields) == 4:
            inclusion_list.append(InclusionListItem(*fields))

    # Get the actual scan number for each mass in the inclusion list
    for scan in range(raw_file.run_header_ex.first_spectrum, raw_file.run_header_ex.last_spectrum):
        # Get the scan filter and event for this scan number        
        scan_filter = raw_file.get_filter_for_scan_number(scan)
        scan_event = raw_file.get_scan_event_for_scan_number(scan)

        # Only consider MS2 scans when looking for the spectrum corr3e
        if scan_filter.ms_order == MsOrderType.Ms2:
            # Get the reaction information in order to get the precursor mass for this spectrum
            reaction = scan_event.get_reaction(0)
            precursor_mass = reaction.precursor_mass
            tolerance = precursor_mass * mass_tolerance

            for item in inclusion_list:
                if item.mass >= precursor_mass - tolerance and item.mass <= precursor_mass + tolerance:
                    item.scan_numer = scan
                    break

    return inclusion_list


def get_chromatogram(raw_file: RawFileAccess, start_scan: int, end_scan: int, output_data: bool):
    """
    Reads the base peak chromatogram for the RAW file
    """
    # Define the settings for getting the Base Peak chromatogram
    settings = ChromatogramTraceSettings(TraceType.BasePeak)

    # Get the chromatogram from the RAW file. 
    data = raw_file.get_chromatogram_data([settings], start_scan, end_scan)

    # Split the data into the chromatograms
    trace = ChromatogramSignal.from_chromatogram_data(data)

    if trace[0].length > 0:
        # Print the chromatogram data (time, intensity values)
        print(f'Base Peak chromatogram ({trace[0].length} points)')

        if output_data:
            for i in range(trace[0].length):
                print(f'  {i} - {trace[0].times[i]}, {trace[0].intensities[i]}')
    print('')


def read_scan_information(raw_file: RawFileAccess, first_scan_number: int, last_scan_number: int, output_data: bool):
    """
    Reads the general scan information for each scan in the RAW file using the scan filter object and also the
    trailer extra data section for that same scan.
    """
    # Read each scan in the RAW File
    for scan in range(first_scan_number, last_scan_number+1):
        # Get the retention time for this scan number.  This is one of two comparable functions that will
        # convert between retention time and scan number.
        time = raw_file.retention_time_from_scan_number(scan)

        # Get the scan filter for this scan number
        # NOTE: A scan filter can also be created from the filter string using the GetFilterFromString in the
        # RawFileAccess.
        scan_filter = raw_file.get_filter_for_scan_number(scan)

        # Get the scan event for this scan number
        scan_event = raw_file.get_scan_event_for_scan_number(scan)
        
        # Get the ionizationMode, MS2 precursor mass, collision energy, and isolation width for each scan
        if scan_filter.ms_order == MsOrderType.Ms2:
            # Get the reaction information for the first precursor
            reaction = scan_event.get_reaction(0)

            precursor_mass = reaction.precursor_mass
            collision_energy = reaction.collision_energy
            isolation_width = reaction.isolation_width
            monoisotopic_mass = 0.0
            master_scan = 0
            ionization_mode = scan_filter.ionization_mode
            order = scan_filter.ms_order

            # Get the trailer extra data for this scan and then look for the monoisotopic m/z value in the 
            # trailer extra data list
            trailer_data = raw_file.get_trailer_extra_information(scan)

            for i in range(trailer_data.length):
                if trailer_data.labels[i] == 'Monoisotopic M/Z:':
                    monoisotopic_mass = float(trailer_data.values[i])
                if trailer_data.labels[i] == 'Master Scan Number:' or trailer_data.labels[i] == 'Master Index:':
                    master_scan = int(trailer_data.values[i])

            if output_data:
                print(f'Scan number {scan} @ time {time} - Master scan = {master_scan}, Ionization mode={ionization_mode}, MS Order={order}, Precursor mass={precursor_mass}, Monoisotopic Mass = {monoisotopic_mass}, Collision energy={collision_energy}, Isolation width={isolation_width}')
        elif scan_filter.ms_order == MsOrderType.Ms:
            scan_dependents = raw_file.get_scan_dependents(scan, 5)
            if scan_dependents is not None:
                print(f'Scan number {scan} @ time {time} - Instrument type={scan_dependents.raw_file_instrument_type}, Number dependent scans={len(scan_dependents.scan_dependent_detail_array)}')


def get_spectrum(raw_file: RawFileAccess, scan_number: int, output_data: bool):
    """
    Gets the spectrum from the RAW file.
    """
    # Get the scan statistics from the RAW file for this scan number
    scan_statistics = raw_file.get_scan_stats_for_scan_number(scan_number)

    # Check to see if the scan has centroid data or profile data.  Depending upon the
    # type of data, different methods will be used to read the data.  While the ReadAllSpectra
    # method demonstrates reading the data using the Scan.FromFile method, generating the
    # Scan object takes more time and memory to do, so that method isn't optimum.
    if scan_statistics.is_centroid_scan and scan_statistics.spectrum_packet_type == SpectrumPacketType.FtCentroid:
        # Get the centroid (label) data from the RAW file for this scan
        centroid_stream = raw_file.get_centroid_stream(scan_number, False)
        print(f'Spectrum (centroid/label) {scan_number} - {centroid_stream.length} points')

        # Print the spectral data (mass, intensity, charge values).  Not all of the information in the high resolution centroid 
        # (label data) object is reported in this example.  Please check the documentation for more information about what is
        # available in high resolution centroid (label) data.
        if output_data:
            for i in range(centroid_stream.length):
                print(f'  {i} - {centroid_stream.masses[i]}, {centroid_stream.intensities[i]}, {centroid_stream.charges[i]}')
        print('')
    else:
        # Get the segmented (low res and profile) scan data
        segmented_scan = raw_file.get_segmented_scan_from_scan_number(scan_number, scan_statistics)
        print(f'Spectrum (normal data) {scan_number} - {len(segmented_scan.positions)} points')

        # Print the spectral data (mass, intensity values)
        if output_data:
            for i in range(len(segmented_scan.positions)):
                print(f'  {i} - {segmented_scan.positions[i]}, {segmented_scan.intensities[i]}')
        print('')


def get_average_spectrum(raw_file: RawFileAccess, first_scan_number: int, last_scan_number: int, output_data: bool):
    """
    Gets the average spectrum from the RAW file.  
    """
    # Create the mass options object that will be used when averaging the scans
    options = raw_file.default_mass_options()
    options.tolerance_units = ToleranceUnits.ppm
    options.tolerance = 5.0

    # Get the scan filter for the first scan.  This scan filter will be used to located
    # scans within the given scan range of the same type
    scan_filter = raw_file.get_filter_for_scan_number(first_scan_number)

    # Get the average mass spectrum for the provided scan range. In addition to getting the
    # average scan using a scan range, the library also provides a similar method that takes
    # a time range.
    average_scan = raw_file.average_scans_in_scan_range(first_scan_number, last_scan_number, scan_filter, options)

    if average_scan.has_centroid_stream:
        print(f'Average spectrum ({average_scan.centroid_scan.length} points)')

        # Print the spectral data (mass, intensity values)
        if output_data:
            for i in range(average_scan.centroid_scan.length):
                print(f'  {average_scan.centroid_scan.masses[0]} {average_scan.centroid_scan.intensities[i]}')

    # This example uses a different method to get the same average spectrum that was calculated in the
    # previous portion of this method.  Instead of passing the start and end scan, a list of scans will
    # be passed to the GetAveragedMassSpectrum function.
    scans = [1, 6, 7, 9, 11, 12, 14]

    average_scan = raw_file.average_scans(scans, options)

    if average_scan.has_centroid_stream:
        print(f'Average spectrum ({average_scan.centroid_scan.length} points)')
        # Print the spectral data (mass, intensity values)
        if output_data:
            for i in range(average_scan.centroid_scan.length):
                print(f'  {average_scan.centroid_scan.masses[0]} {average_scan.centroid_scan.intensities[i]}')
    print('')


def read_all_spectra(raw_file: RawFileAccess, first_scan_number: int, last_scan_number: int, output_data: bool):
    """
    Read all spectra in the RAW file.
    """
    for scan_number in range(first_scan_number, last_scan_number + 1):
        try:
            # Get the scan filter for the spectrum
            scan_filter = raw_file.get_filter_for_scan_number(scan_number)

            if scan_filter is None or str(scan_filter) == '':
                continue

            # Get the scan from the RAW file.  This method uses the Scan.FromFile method which returns a
            # Scan object that contains both the segmented and centroid (label) data from an FTMS scan
            # or just the segmented data in non-FTMS scans.  The GetSpectrum method demonstrates an
            # alternative method for reading scans.
            scan = Scan.from_file(raw_file, scan_number)

            # If that scan contains FTMS data then Centroid stream will be populated so check to see if it is present.
            label_size = 0
            if scan.has_centroid_stream:
                label_size = scan.centroid_scan.length

            # For non-FTMS data, the preferred data will be populated
            data_size = len(scan.preferred_masses)
            if output_data:
                print(f'Spectrum {scan_number} - {scan_filter}: normal {data_size}, label {label_size} points')

        except Exception as ex:
            print(f'Error reading spectrum {scan_number} - {ex}')


def calculate_mass_precision(raw_file: RawFileAccess, scan_number: int):
    """
    Calculates the mass precision for a spectrum.  
    """
    # Get the scan from the RAW file
    scan = Scan.from_file(raw_file, scan_number)

    # Get the scan event and from the scan event get the analyzer type for this scan
    scan_event = raw_file.get_scan_event_for_scan_number(scan_number)

    # Get the trailer extra data to get the ion time for this file
    log_entry = raw_file.get_trailer_extra_information(scan_number)

    trailer_headings = list()
    trailer_values = list()
    for i in range(log_entry.length):
        trailer_headings.append(log_entry.labels[i])
        trailer_values.append(log_entry.values[i])

    # Create the mass precision estimate object
    precision_estimate = PrecisionEstimate()

    # Get the ion time from the trailer extra data values
    ion_time = precision_estimate.get_ion_time(scan_event.mass_analyzer, scan, trailer_headings, trailer_values)

    # Calculate the mass precision for the scan
    list_results = precision_estimate.get_mass_precision_estimate(scan, scan_event.mass_analyzer, ion_time, raw_file.run_header.mass_resolution)

    # Output the mass precision results
    if len(list_results) > 0:
        print('Mass Precision Results:')
        for result in list_results:
            print(f'Mass {result.mass}, mmu = {result.mass_accuracy_in_mmu}, ppm = {result.mass_accuracy_in_ppm}')


def analyze_all_scans(raw_file: RawFileAccess, first_scan_number: int, last_scan_number: int):
    """
    Reads all of the scans in the RAW and looks for out of order data
    """
    # Test the preferred (normal) data and centroid (high resolution/label) data
    failed_centroid = 0
    failed_preferred = 0

    for scan_number in range(first_scan_number, last_scan_number+1):
        # Get each scan from the RAW file
        scan = Scan.from_file(raw_file, scan_number)

        # Check to see if the RAW file contains label (high-res) data and if it is present
        # then look for any data that is out of order
        if scan.has_centroid_stream:
            if scan.centroid_scan.length:
                current_mass = scan.centroid_scan.masses[0]

                for index in range(1, scan.centroid_scan.length):
                    if scan.centroid_scan.masses[index] > current_mass:
                        current_mass = scan.centroid_scan.masses[index]
                    else:
                        if failed_centroid == 0:
                            print(f'First failure: Failed in scan data at: Scan: {scan_number} Mass: {current_mass}')
                        failed_centroid += 1

        # Check the normal (non-label) data in the RAW file for any out-of-order data
        if len(scan.preferred_masses) > 0:
            current_mass = scan.preferred_masses[0]
            for index in range(1, len(scan.preferred_masses)):
                if scan.preferred_masses[index] > current_mass:
                    current_mass = scan.preferred_masses[index]
                else:
                    if failed_preferred == 0:
                        print(f'First failure: Failed in scan data at: Scan: {scan_number} Mass: {current_mass}')
                    failed_preferred += 1

    # Display a message indicating if any of the scans had data that was "out of order"
    print('')
    if failed_preferred == 0 and failed_centroid == 0:
        print('Analysis completed: No out of order data found')
    else:
        print(f'Analysis completed: Preferred data failed: {failed_preferred} Centroid data failed: {failed_centroid}')


if __name__ == '__main__':
    # This local variables controls if certain operations are performed. Change any of them to true to read and output that
    # information section from the RAW file.
    analyze_scans = True#False
    average_scans = False
    do_calculate_mass_precision = False
    do_get_chromatogram = True
    do_get_inclusion_exclusion_list = True#False
    get_status_log = True
    get_trailer_extra = True
    read_all_scans = True
    do_read_scan_information = False
    read_spectrum = True

    try:
        # Check to see if the RAW file name was supplied as an argument to the program
        filename = ''

        args = sys.argv[1:]
        if len(args) > 0:
            filename = args[0]

        if filename == '':
            print('No RAW file specified!')
            sys.exit(0)

        # Check to see if the specified RAW file exists
        if not os.path.exists(filename):
            print(f"The file doesn't exist in the specified location - {filename}")
            sys.exit(0)

        # Create the RawFileAccess object for accessing the RAW file
        raw_file = RawFileReaderAdapter.file_factory(filename)

        if not raw_file.is_open or raw_file.is_error:
            print('Unable to access the RAW file using the RawFileReader class!')
            sys.exit(0)

        # Check if the RAW file is being acquired
        if raw_file.in_acquisition:
            print(f'RAW file still being acquired - {filename}')
            sys.exit(0)

        # Get the number of instruments (controllers) present in the RAW file and set the 
        # selected instrument to the MS instrument, first instance of it
        print(f'The RAW file has data from {raw_file.instrument_count} instruments')

        raw_file.select_instrument(Device.MS, 1)

        # Get the first and last scan from the RAW file
        first_scan_number = raw_file.run_header_ex.first_spectrum
        last_scan_number = raw_file.run_header_ex.last_spectrum

        # get the start and end time from the RAW file
        start_time = raw_file.run_header_ex.start_time
        end_time = raw_file.run_header_ex.end_time

        # Get some information from the header portions of the RAW file and display that information.
        # The information is general information pertaining to the RAW file.
        print('General File Information:')
        print(f'   RAW file: {raw_file.file_name}')
        #print(f'   RAW file version: {raw_file.file_header.revision}')
        #print(f'   Creation date: {raw_file.file_header.creation_date}')
        #print(f'   Operator: {raw_file.file_header.who_created_id}')
        print(f'   Number of instruments: {raw_file.instrument_count}')
        #print(f'   Description: {raw_file.file_header.file_description}')
        print(f'   Instrument model: {raw_file.get_instrument_data().model}')
        print(f'   Instrument name: {raw_file.get_instrument_data().name}')
        print(f'   Serial number: {raw_file.get_instrument_data().serial_number}')
        print(f'   Software version: {raw_file.get_instrument_data().software_version}')
        print(f'   Firmware version: {raw_file.get_instrument_data().hardware_version}')
        print(f'   Units: {raw_file.get_instrument_data().units}')
        print(f'   Mass resolution: {raw_file.run_header_ex.mass_resolution}')
        print(f'   Number of scans: {raw_file.run_header_ex.spectra_count}')
        print(f'   Scan range: {first_scan_number} - {last_scan_number}')
        print(f'   Time range: {start_time} - {end_time}')
        print(f'   Mass range: {raw_file.run_header_ex.low_mass} - {raw_file.run_header_ex.high_mass}')
        print('')

        # Get information related to the sample that was processed
        print('Sample Information:')
        print(f'   Sample name: {raw_file.sample_information.sample_name}')
        print(f'   Sample id: {raw_file.sample_information.sample_id}')
        print(f'   Sample type: {raw_file.sample_information.sample_type}')
        print(f'   Sample comment: {raw_file.sample_information.comment}')
        print(f'   Sample vial: {raw_file.sample_information.vial}')
        print(f'   Sample volume: {raw_file.sample_information.sample_volume}')
        print(f'   Sample injection volume: {raw_file.sample_information.injection_volume}')
        print(f'   Sample row number: {raw_file.sample_information.row_number}')
        print(f'   Sample dilution factor: {raw_file.sample_information.dilution_factor}')
        print('')

        # Display all of the trailer extra data fields present in the RAW file
        if get_trailer_extra:
            list_trailer_extra_fields(raw_file)

        # Get the status log items
        if get_status_log:
            list_status_log(raw_file, first_scan_number, last_scan_number)

        # Get the inclusion/exclusion list
        if do_get_inclusion_exclusion_list:
            inclusion_list = get_inclusion_exclusion_list(raw_file, 1e-5)

            # Output the saved inclusion/exclusion list
            count = 0

            for item in inclusion_list:
                print(f'  {count} - {item.descriptor}, {item.mass}, {item.threshold}, {item.scan_number}')
                count += 1
            print('')

        # Get the number of filters present in the RAW file
        number_filters = len(raw_file.get_filters())

        # Get the scan filter for the first and last spectrum in the RAW file
        first_filter = raw_file.get_filter_for_scan_number(first_scan_number)
        last_filter = raw_file.get_filter_for_scan_number(last_scan_number)

        print('Filter Information:')
        print(f'   Scan filter (first scan): {str(first_filter)}')
        print(f'   Scan filter (last scan): {str(last_filter)}')
        print(f'   Total number of filters:{number_filters}')
        print('')

        # Get the BasePeak chromatogram for the MS data
        if do_get_chromatogram:
            get_chromatogram(raw_file, first_scan_number, last_scan_number, True)

        # Read the scan information for each scan in the RAW file
        if do_read_scan_information:
            read_scan_information(raw_file, first_scan_number, last_scan_number, True)

        # Get a spectrum from the RAW file.
        if read_spectrum:
            get_spectrum(raw_file, first_scan_number, False)

        # Get a average spectrum from the RAW file for the first 15 scans in the file.  
        if average_scans:
            get_average_spectrum(raw_file, 1, 15, True)

        # Read each spectrum
        if read_all_scans:
            read_all_spectra(raw_file, first_scan_number, last_scan_number, True)

        # Calculate the mass precision for a spectrum
        if do_calculate_mass_precision:
            calculate_mass_precision(raw_file, 1)

        # Check all of the scans for out of order data.  This method isn't enabled by
        # default because it is very, very time consuming.  If you would like to 
        # call this method change the value of _analyzeScans to true.
        if analyze_scans:
            analyze_all_scans(raw_file, first_scan_number, last_scan_number)

        # Close (dispose) the RAW file
        print('')
        print(f'Closing {filename}')
        raw_file.dispose()        

    except Exception as e:
        print(e)
        print(traceback.print_exc())
        pass


from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from typing import List


class ChromatogramData(NetWrapperBase):

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.ChromatogramSignal

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    @property
    def positions_array(self) -> List[List[float]]:
        """
        Gets Times in minutes for each chromatogram
        """
        return [[n for n in sublist] for sublist in self._get_wrapped_object_().PositionsArray]

    @property
    def scan_numbers_array(self) -> List[List[int]]:
        """
        Gets Scan numbers for data points in each chromatogram
        """
        return [[n for n in sublist] for sublist in self._get_wrapped_object_().ScanNumbersArray]

    @property
    def intensities_array(self) -> List[List[float]]:
        """
        Gets Intensities for each chromatogram
        """
        return [[n for n in sublist] for sublist in self._get_wrapped_object_().IntensitiesArray]

    @property
    def length(self) -> int:
        """
        Gets The number of chromatograms in this object
        """
        return self._get_wrapped_object_().Length



from fisher_py.data.ft_average_options import FtAverageOptions
from fisher_py.data.raw_file_classification import RawFileClassification
from fisher_py.data.file_type import FileType
from fisher_py.data.file_header import FileHeader
from fisher_py.data.source_fragmentation_info_valid_type import SourceFragmentationInfoValidType
from fisher_py.data.scan_dependent_details import ScanDependentDetails
from fisher_py.data.filter_accurate_mass import FilterAccurateMass
from fisher_py.data.scan_filter import ScanFilter
from fisher_py.data.tolerance_units import ToleranceUnits
from fisher_py.data.device import Device
from fisher_py.data.peak_options import PeakOptions
from fisher_py.data.common_core_data_object import CommonCoreDataObject
from fisher_py.data.file_error import FileError
from fisher_py.data.sequence_info import SequenceInfo
from fisher_py.data.sequence_file_writer import SequenceFileWriter
from fisher_py.data.scan_event import ScanEvent



from fisher_py.net_wrapping import NetWrapperBase
from fisher_py.data import ToleranceUnits


class WrappedRunHeader(NetWrapperBase):
    """
    Information about the file stream
    """

    def __init__(self):
        super().__init__()
        # can only be used for wrapping (no Python side instantiation

    @property
    def spectra_count(self) -> int:
        """
        Gets the count of recorded spectra
        """
        return self._get_wrapped_object_().SpectraCount

    @property
    def high_mass(self) -> float:
        """
        Gets the highest recorded mass in file
        """
        return self._get_wrapped_object_().HighMass

    @property
    def low_mass(self) -> float:
        """
        Gets the lowest recorded mass in file
        """
        return self._get_wrapped_object_().LowMass

    @property
    def end_time(self) -> float:
        """
        Gets the time of last scan in file
        """
        return self._get_wrapped_object_().EndTime

    @property
    def start_time(self) -> float:
        """
        Gets the time of first scan in file
        """
        return self._get_wrapped_object_().StartTime

    @property
    def last_spectrum(self) -> int:
        """
        Gets the last spectrum (scan) number. If this is less than 1, then there are
        no scans acquired yet.
        """
        return self._get_wrapped_object_().LastSpectrum

    @property
    def first_spectrum(self) -> int:
        """
        Gets the first spectrum (scan) number (typically 1).
        """
        return self._get_wrapped_object_().FirstSpectrum

    @property
    def comment_2(self) -> str:
        """
        Gets the second comment about this data stream.
        """
        return self._get_wrapped_object_().Comment2

    @property
    def comment_1(self) -> str:
        """
        Gets the first comment about this data stream.
        """
        return self._get_wrapped_object_().Comment1

    @property
    def max_intensity(self) -> float:
        """
        Gets the max intensity.
        """
        return self._get_wrapped_object_().MaxIntensity

    @property
    def filter_mass_precision(self) -> int:
        """
        Gets the number of digits of precision suggested for formatting masses in the
        filters.
        """
        return self._get_wrapped_object_().FilterMassPrecision

    @property
    def expected_run_time(self) -> float:
        """
        Gets the expected data acquisition time.
        """
        return self._get_wrapped_object_().ExpectedRunTime

    @property
    def mass_resolution(self) -> float:
        """
        Gets the mass resolution of this instrument.
        """
        return self._get_wrapped_object_().MassResolution

    @property
    def in_acquisition(self) -> int:
        """
        Gets a value indicating whether this file is being created. 0: File is complete.
        All other positive values: The file is in acquisition. Negative values are undefined.
        """
        return self._get_wrapped_object_().InAcquisition

    @property
    def trailer_extra_count(self) -> int:
        """
        Gets the count of "trailer extra" records. Typically, same as the count of scans.
        """
        return self._get_wrapped_object_().TrailerExtraCount

    @property
    def trailer_scan_event_count(self) -> int:
        """
        Gets the count of "scan events"
        """
        return self._get_wrapped_object_().TrailerScanEventCount

    @property
    def error_log_count(self) -> int:
        """
        Gets the count of error log entries
        """
        return self._get_wrapped_object_().ErrorLogCount

    @property
    def tune_data_count(self) -> int:
        """
        Gets the count of tune data entries
        """
        return self._get_wrapped_object_().TuneDataCount

    @property
    def status_log_count(self) -> int:
        """
        Gets the count of status log entries
        """
        return self._get_wrapped_object_().StatusLogCount

    @property
    def tolerance_unit(self) -> ToleranceUnits:
        """
        Gets the tolerance units
        """
        return ToleranceUnits(self._get_wrapped_object_().ToleranceUnit)

    @property
    def max_integrated_intensity(self) -> float:
        """
        Gets the max integrated intensity.
        """
        return self._get_wrapped_object_().MaxIntegratedIntensity


import enum


class MassAnalyzerType(enum.Enum):
    """
    Specifies type of mass analyzer in scans.
    """
    MassAnalyzerITMS = 0
    MassAnalyzerTQMS = 1
    MassAnalyzerSQMS = 2
    MassAnalyzerTOFMS = 3
    MassAnalyzerFTMS = 4
    MassAnalyzerSector = 5
    Any = 6



from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher


class FtAverageOptions(NetWrapperBase):
    """
    Options which can be used to control the Ft / Orbitrap averaging
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Interfaces.FtAverageOptions

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    @property
    def max_charge_determinations(self) -> int:
        """
        Gets or sets the maximum number of ions which are sent to the charge pattern
        calculation (starting from most intense)
        """
        return self._get_wrapped_object_().MaxChargeDeterminations

    @max_charge_determinations.setter
    def max_charge_determinations(self, value: int):
        """
        Gets or sets the maximum number of ions which are sent to the charge pattern
        calculation (starting from most intense)
        """
        assert type(value) is int
        self._get_wrapped_object_().MaxChargeDeterminations = value

    @property
    def merge_in_parallel(self) -> bool:
        """
        Gets or sets a value indicating whether parallel code may be used for resampling
        and merging scans. Tuning option: Permit separate threads to be used for resampling
        profiles.
        """
        return self._get_wrapped_object_().MergeInParallel

    @merge_in_parallel.setter
    def merge_in_parallel(self, value: bool):
        """
        Gets or sets a value indicating whether parallel code may be used for resampling
        and merging scans. Tuning option: Permit separate threads to be used for resampling
        profiles.
        """
        assert type(value) is bool
        self._get_wrapped_object_().MergeInParallel = value

    @property
    def max_scans_merged(self) -> int:
        """
        Gets or sets the maximum number of scans which can be merged at once. This feature
        is currently not yet implemented, and the value is ignored. When MergeInParallel
        is enabled: this restricts the number of scans which are merged in each group.
        Setting this too large may result in more memory allocation for "arrays of results
        to merge" Default: 10
        """
        return self._get_wrapped_object_().MaxScansMerged

    @max_scans_merged.setter
    def max_scans_merged(self, value: int):
        """
        Gets or sets the maximum number of scans which can be merged at once. This feature
        is currently not yet implemented, and the value is ignored. When MergeInParallel
        is enabled: this restricts the number of scans which are merged in each group.
        Setting this too large may result in more memory allocation for "arrays of results
        to merge" Default: 10
        """
        assert type(value) is int
        self._get_wrapped_object_().MaxScansMerged = value

    @property
    def merge_task_batching(self) -> int:
        """
        Gets or sets the minimum number of Re-sample tasks per thread. Tuning parameter
        when MergeInParallel is set. Each scan is analyzed: Determining mass regions
        which contain non-zero data, and re-sampling the intensity data aligned to a
        set of output bins. After all scans have been re-sampled, the re-sampled data
        has to be merged into the final output. Creating re-sampled data for profiles
        is a fairly fast task. It may be inefficient to queue workers to created the
        merged data for each scan in the batch. Setting this >1 will reduce threading
        overheads, when averaging small batches of scans with low intensity peaks. Default:
        2. This feature only affects the re-sampling, as the final merge of the re-sampled
        data is single threaded.
        """
        return self._get_wrapped_object_().MergeTaskBatching

    @merge_task_batching.setter
    def merge_task_batching(self, value: int):
        """
        Gets or sets the minimum number of Re-sample tasks per thread. Tuning parameter
        when MergeInParallel is set. Each scan is analyzed: Determining mass regions
        which contain non-zero data, and re-sampling the intensity data aligned to a
        set of output bins. After all scans have been re-sampled, the re-sampled data
        has to be merged into the final output. Creating re-sampled data for profiles
        is a fairly fast task. It may be inefficient to queue workers to created the
        merged data for each scan in the batch. Setting this >1 will reduce threading
        overheads, when averaging small batches of scans with low intensity peaks. Default:
        2. This feature only affects the re-sampling, as the final merge of the re-sampled
        data is single threaded.
        """
        assert type(value) is int
        self._get_wrapped_object_().MergeTaskBatching = value

    @property
    def use_noise_table_when_available(self) -> bool:
        """
        Gets or sets a value indicating whether to use the noise and baseline table.
        When set: The averaging algorithm calculates average noise based on a noise table
        obtained (separately) from the raw file. The "IRawData" interface doe not have
        methods to obtain this "noise and baseline table" from the raw file. So: The
        scan averaging algorithm (by default) uses noise information saved with centroid
        peaks when calculating the averaged noise. This option is only effective when
        data is read via the IRawDataPlus interface.
        """
        return self._get_wrapped_object_().UseNoiseTableWhenAvailable

    @use_noise_table_when_available.setter
    def use_noise_table_when_available(self, value: bool):
        """
        Gets or sets a value indicating whether to use the noise and baseline table.
        When set: The averaging algorithm calculates average noise based on a noise table
        obtained (separately) from the raw file. The "IRawData" interface doe not have
        methods to obtain this "noise and baseline table" from the raw file. So: The
        scan averaging algorithm (by default) uses noise information saved with centroid
        peaks when calculating the averaged noise. This option is only effective when
        data is read via the IRawDataPlus interface.
        """
        assert type(value) is bool
        self._get_wrapped_object_().UseNoiseTableWhenAvailable = value




from fisher_py.net_wrapping import ThermoFisher
from fisher_py.utils import is_number
from fisher_py.data.business import Range, MassOptions

_range_factory = ThermoFisher.CommonCore.Data.Business.RangeFactory


class RangeFactor(object):
    """
    Factory to produce immutable ranges of double
    """

    @staticmethod
    def create(low: float, high: float) -> Range:
        """
        Create an immutable (constant) range from low and high.
        
        Parameters:
          low:
            The low.
        
          high:
            The high.
        
        Returns:
            The range.
        """
        assert is_number(low)
        assert is_number(high)
        return Range._get_wrapper_(_range_factory.Create(float(low), float(high)))

    @staticmethod
    def create_from_center_and_delta(center: float, delta: float) -> Range:
        """
        Create an immutable (constant) range from center and delta, such that the range
        is center +/- delta.
        
        Parameters:
          center:
            The center.
        
          delta:
            The delta.
        
        Returns:
            The ThermoFisher.CommonCore.Data.Business.Range.
        """
        assert is_number(center)
        assert is_number(delta)
        return Range._get_wrapper_(_range_factory.CreateFromCenterAndDelta(float(center), float(delta)))

    @staticmethod
    def create_from_range_and_tolerance(from_: Range, arg) -> Range:
        """
        Construct a range from another range, adding a tolerance if ends are the same
        """
        assert type(from_) is Range

        if type(arg) is MassOptions:
            return Range._get_wrapper_(_range_factory.CreateFromRangeAndTolerance(from_._get_wrapped_object_(), arg._get_wrapped_object_()))
        elif is_number(arg):
            return Range._get_wrapper_(_range_factory.CreateFromRangeAndTolerance(from_._get_wrapped_object_(), float(arg)))
        
        raise ValueError('Unable to create range.')



from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data import PeakOptions


class LabelPeak(NetWrapperBase):

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.LabelPeak

    def __init__(self):
        super().__init__()
        self._mass_options = None

    @property
    def mass(self) -> float:
        """
        Gets or sets mass.
        """
        return self._get_wrapped_object_().Mass

    @mass.setter
    def mass(self, value: float):
        """
        Gets or sets mass.
        """
        assert type(value) is float
        self._get_wrapped_object_().Mass = value

    @property
    def intensity(self) -> float:
        """
        Gets or sets intensity.
        """
        return self._get_wrapped_object_().Intensity

    @intensity.setter
    def intensity(self, value: float):
        """
        Gets or sets intensity.
        """
        assert type(value) is float
        self._get_wrapped_object_().Intensity = value

    @property
    def resolution(self) -> float:
        """
        Gets or sets resolution.
        """
        return self._get_wrapped_object_().Resolution

    @resolution.setter
    def resolution(self, value: float):
        """
        Gets or sets resolution.
        """
        assert type(value) is float
        self._get_wrapped_object_().Resolution = value

    @property
    def baseline(self) -> float:
        """
        Gets or sets base Line.
        """
        return self._get_wrapped_object_().Baseline

    @baseline.setter
    def baseline(self, value: float):
        """
        Gets or sets base Line.
        """
        assert type(value) is float
        self._get_wrapped_object_().Baseline = value

    @property
    def noise(self) -> float:
        """
        Gets or sets noise.
        """
        return self._get_wrapped_object_().Noise

    @noise.setter
    def noise(self, value: float):
        """
        Gets or sets noise.
        """
        assert type(value) is float
        self._get_wrapped_object_().Noise = value

    @property
    def charge(self) -> float:
        """
        Gets or sets charge.
        """
        return self._get_wrapped_object_().Charge

    @charge.setter
    def charge(self, value: float):
        """
        Gets or sets charge.
        """
        assert type(value) is float
        self._get_wrapped_object_().Charge = value

    @property
    def flag(self) -> PeakOptions:
        """
        Gets or sets Peak Options Flag.
        """
        return PeakOptions(self._get_wrapped_object_().Flag)

    @flag.setter
    def flag(self, value: PeakOptions):
        """
        Gets or sets Peak Options Flag.
        """
        assert type(value) is PeakOptions
        self._get_wrapped_object_().Flag = value.value

    @property
    def signal_to_noise(self) -> float:
        """
        Gets or sets the signal to noise.
        """
        return self._get_wrapped_object_().SignalToNoise

    @signal_to_noise.setter
    def signal_to_noise(self, value: float):
        """
        Gets or sets the signal to noise.
        """
        assert type(value) is float
        self._get_wrapped_object_().SignalToNoise = value



from fisher_py.utils import is_number
from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data.business import Range


class MassToFrequencyConverter(NetWrapperBase):
    """
    Class which includes the coefficients for mass calibration from a particular
    scan, and means to convert between mass and frequency
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.MassToFrequencyConverter

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    @property
    def coefficient_1(self) -> float:
        """
        Gets or sets the coefficient 1.
        """
        return self._get_wrapped_object_().Coefficient1

    @coefficient_1.setter
    def coefficient_1(self, value: float):
        """
        Gets or sets the coefficient 1.
        """
        assert is_number(value)
        self._get_wrapped_object_().Coefficient1 = float(value)

    @property
    def coefficient_2(self) -> float:
        """
        Gets or sets the coefficient 2.
        """
        return self._get_wrapped_object_().Coefficient2

    @coefficient_2.setter
    def coefficient_2(self, value: float):
        """
        Gets or sets the coefficient 2.
        """
        assert is_number(value)
        self._get_wrapped_object_().Coefficient2 = float(value)

    @property
    def coefficient_3(self) -> float:
        """
        Gets or sets the coefficient 3.
        """
        return self._get_wrapped_object_().Coefficient3

    @coefficient_3.setter
    def coefficient_3(self, value: float):
        """
        Gets or sets the coefficient 3.
        """
        assert is_number(value)
        self._get_wrapped_object_().Coefficient3 = float(value)

    @property
    def base_frequency(self) -> float:
        """
        Gets or sets the base frequency.
        """
        return self._get_wrapped_object_().BaseFrequency

    @base_frequency.setter
    def base_frequency(self, value: float):
        """
        Gets or sets the base frequency.
        """
        assert is_number(value)
        self._get_wrapped_object_().BaseFrequency = float(value)

    @property
    def delta_frequency(self) -> float:
        """
        Gets or sets the delta frequency.
        """
        return self._get_wrapped_object_().DeltaFrequency

    @delta_frequency.setter
    def delta_frequency(self, value: float):
        """
        Gets or sets the delta frequency.
        """
        assert is_number(value)
        self._get_wrapped_object_().DeltaFrequency = float(value)

    @property
    def highest_mass(self) -> float:
        """
        Gets or sets the highest mass.
        """
        return self._get_wrapped_object_().HighestMass

    @highest_mass.setter
    def highest_mass(self, value: float):
        """
        Gets or sets the highest mass.
        """
        assert is_number(value)
        self._get_wrapped_object_().HighestMass = float(value)

    @property
    def segment_range(self) -> Range:
        """
        Gets or sets the (largest) segment range of the scans processed.
        """
        return self._get_wrapped_object_().SegmentRange

    @segment_range.setter
    def segment_range(self, value: Range):
        """
        Gets or sets the (largest) segment range of the scans processed.
        """
        assert is_number(value)
        self._get_wrapped_object_().SegmentRange = float(value)

    def convert_frequence_to_mass(self, sample: int) -> float:
        """
        Converts the given frequency to it's corresponding mass.
        
        Parameters:
        sample:
        sample number to convert
        
        Returns:
        converted mass.
        """
        assert type(sample) is int
        return self._get_wrapped_object().ConvertFrequenceToMass(sample)

    def convert_mass_to_frequency(self, mass: float) -> float:
        """
        Converts the given mass to frequency.
        
        Parameters:
        mass:
        The mass to convert
        
        Returns:
        converted frequency
        """
        assert is_number(mass)
        return self._get_wrapped_object().ConvertMassToFrequency(float(mass))




from fisher_py.net_wrapping import ThermoFisher
from fisher_py.raw_file_reader import RawFileAccess
from fisher_py.exceptions import RawFileException


_net_raw_file_reader_adapter = ThermoFisher.CommonCore.RawFileReader.RawFileReaderAdapter

class RawFileReaderAdapter(object):
    """
    Utility to load raw files
    """

    @staticmethod
    def file_factory(raw_file: str):
        """
        Create an IRawDataExtended interface to read data from a raw file

        :param raw_file: Path to raw file
        :return: Returns raw file
        """
        return RawFileAccess(raw_file)


if __name__ == '__main__':
    raw_file_path = r"D:\Nextcloud\Shared\MS\data\00_raw\00_corrected_ion_mass\20210407_15mer_bleomycin.raw"
    raw_file = RawFileReaderAdapter.file_factory(raw_file_path)
    print(raw_file)


import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='fisher_py',
    version='1.0.12',
    author='ethz-institute-of-microbiology',
    author_email='dominik.werner@live.com',
    description='This python module allows to extract data from the RAW-file-format produces by devices from Thermo Fisher Scientific.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/ethz-institute-of-microbiology/fisher_py',
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        'pythonnet>=2.5.1',
        'numpy>=1.17.0'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



from fisher_py.net_wrapping import NetWrapperBase
from fisher_py.data import FileType
from fisher_py.utils import datetime_net_to_py, datetime_py_to_net
from datetime import datetime


class FileHeader(NetWrapperBase):
    @property
    def who_created_id(self) -> str:
        """
        Gets or sets the creator Id. The creator Id is the full text user name of the
        user when the file is created.
        """
        return self._get_wrapped_object_().WhoCreatedId

    @who_created_id.setter
    def who_created_id(self, value: str):
        """
        Gets or sets the creator Id. The creator Id is the full text user name of the
        user when the file is created.
        """
        assert type(value) is str
        self._get_wrapped_object_().WhoCreatedId = value

    @property
    def who_created_logon(self) -> str:
        """
        Gets or sets the creator Login name. The creator login name is the user name
        of the user when the file is created, as entered at the "user name, password"
        screen in windows.
        """
        return self._get_wrapped_object_().WhoCreatedLogon

    @who_created_logon.setter
    def who_created_logon(self, value: str):
        """
        Gets or sets the creator Login name. The creator login name is the user name
        of the user when the file is created, as entered at the "user name, password"
        screen in windows.
        """
        assert type(value) is str
        self._get_wrapped_object_().WhoCreatedLogon = value

    @property
    def who_modified_id(self) -> str:
        """
        Gets or sets the creator Id. The creator Id is the full text user name of the
        user when the file is created.
        """
        return self._get_wrapped_object_().WhoModifiedId

    @who_modified_id.setter
    def who_modified_id(self, value: str):
        """
        Gets or sets the creator Id. The creator Id is the full text user name of the
        user when the file is created.
        """
        assert type(value) is str
        self._get_wrapped_object_().WhoModifiedId = value

    @property
    def who_modified_logon(self) -> str:
        """
        Gets or sets the creator Login name. The creator login name is the user name
        of the user when the file is created, as entered at the "user name, password"
        screen in windows.
        """
        return self._get_wrapped_object_().WhoModifiedLogon

    @who_modified_logon.setter
    def who_modified_logon(self, value: str):
        """
        Gets or sets the creator Login name. The creator login name is the user name
        of the user when the file is created, as entered at the "user name, password"
        screen in windows.
        """
        assert type(value) is str
        self._get_wrapped_object_().WhoModifiedLogon = value

    @property
    def file_type(self) -> FileType:
        """
        Gets or sets the type of the file. If the file is not recognized, the value of
        the FileType will be set to "Not Supported"
        """
        return FileType(self._get_wrapped_object_().FileType)

    @file_type.setter
    def file_type(self, value: FileType):
        """
        Gets or sets the type of the file. If the file is not recognized, the value of
        the FileType will be set to "Not Supported"
        """
        assert type(value) is FileType
        self._get_wrapped_object_().FileType = value.value

    @property
    def revision(self) -> int:
        """
        Gets or sets the file format revision Note: this does not refer to revisions
        of the content. It defines revisions of the binary files structure.
        """
        return self._get_wrapped_object_().Revision

    @revision.setter
    def revision(self, value: int):
        """
        Gets or sets the file format revision Note: this does not refer to revisions
        of the content. It defines revisions of the binary files structure.
        """
        assert type(value) is int
        self._get_wrapped_object_().Revision = value

    @property
    def creation_date(self) -> datetime:
        """
        Gets or sets the file creation date.
        """
        return datetime_net_to_py(self._get_wrapped_object_().CreationDate)

    @creation_date.setter
    def creation_date(self, value: datetime):
        """
        Gets or sets the file creation date.
        """
        assert type(value) is datetime
        self._get_wrapped_object_().CreationDate = datetime_py_to_net(value)

    @property
    def modified_date(self) -> datetime:
        """
        Gets or sets the modified date. File changed audit information (most recent change)
        
        Value:
        The modified date.
        """
        return datetime_net_to_py(self._get_wrapped_object_().ModifiedDate)

    @modified_date.setter
    def modified_date(self, value: datetime):
        """
        Gets or sets the modified date. File changed audit information (most recent change)
        
        Value:
        The modified date.
        """
        assert type(value) is datetime
        self._get_wrapped_object_().ModifiedDate = datetime_py_to_net(value)

    @property
    def number_of_times_modified(self) -> int:
        """
        Gets or sets the number of times modified.
        
        Value:
        The number of times the file has been modified.
        """
        return self._get_wrapped_object_().NumberOfTimesModified

    @number_of_times_modified.setter
    def number_of_times_modified(self, value: int):
        """
        Gets or sets the number of times modified.
        
        Value:
        The number of times the file has been modified.
        """
        assert type(value) is int
        self._get_wrapped_object_().NumberOfTimesModified = value

    @property
    def number_of_times_calibrated(self) -> int:
        """
        Gets or sets the number of times calibrated.
        
        Value:
        The number of times calibrated.
        """
        return self._get_wrapped_object_().NumberOfTimesCalibrated

    @number_of_times_calibrated.setter
    def number_of_times_calibrated(self, value: int):
        """
        Gets or sets the number of times calibrated.
        
        Value:
        The number of times calibrated.
        """
        assert type(value) is int
        self._get_wrapped_object_().NumberOfTimesCalibrated = value

    @property
    def file_description(self) -> str:
        """
        Gets or sets the file description. User's narrative description of the file,
        512 unicode characters (1024 bytes)
        
        Value:
        The file description.
        """
        return self._get_wrapped_object_().FileDescription

    @file_description.setter
    def file_description(self, value: str):
        """
        Gets or sets the file description. User's narrative description of the file,
        512 unicode characters (1024 bytes)
        
        Value:
        The file description.
        """
        assert type(value) is str
        self._get_wrapped_object_().FileDescription = value




from fisher_py.net_wrapping import NetWrapperBase
from fisher_py.data import FileHeader, FileError, SequenceInfo
from fisher_py.data.business import SampleInformation, BracketType
from typing import List


class SequenceFileWriter(NetWrapperBase):
    
    def __init__(self):
        super().__init__()
        self._file_header = None
        self._file_error = None
        self._sequence_info = None

    @property
    def file_header(self) -> FileHeader:
        """
        Gets the file header for the sequence
        """
        if self._file_header is None:
            self._file_header = FileHeader._get_wrapper_(self._get_wrapped_object_().FileHeader)

        return self._file_header

    @property
    def file_error(self) -> FileError:
        """
        Gets the file error state.
        """
        if self._file_error is None:
            self._file_error = FileError._get_wrapper_(self._get_wrapped_object_().FileError)

        return self._file_error

    @property
    def is_error(self) -> bool:
        """
        Gets a value indicating whether the last file operation caused an error
        """
        return self._get_wrapped_object_().IsError

    @property
    def info(self) -> SequenceInfo:
        """
        Gets or sets additional information about a sequence
        """
        if self._sequence_info is None:
            self._sequence_info = SequenceInfo._get_wrapper_(self._get_wrapped_object_().Info)
        return self._sequence_info

    @info.setter
    def info(self, value: SequenceInfo):
        """
        Gets or sets additional information about a sequence
        """
        assert type(value) is SequenceInfo
        self._sequence_info = value
        self._get_wrapped_object_().Info = value._get_wrapped_object_()

    @property
    def samples(self) -> List[SampleInformation]:
        """
        Gets the set of samples in the sequence
        """
        return [SampleInformation._get_wrapper_(s) for s in self._get_wrapped_object_().Samples]

    @property
    def file_name(self) -> str:
        """
        Gets the name of the sequence file.
        """
        return self._get_wrapped_object_().FileName

    @property
    def bracket(self) -> BracketType:
        """
        Gets or sets the sequence bracket type. This determines which groups of samples
        use the same calibration curve.
        """
        return BracketType(self._get_wrapped_object_().Bracket)

    @bracket.setter
    def bracket(self, value: BracketType):
        """
        Gets or sets the sequence bracket type. This determines which groups of samples
        use the same calibration curve.
        """
        assert type(value) is BracketType
        self._get_wrapped_object_().Bracket = value.value

    @property
    def tray_configuration(self) -> str:
        """
        Gets or sets a description of the auto-sampler tray
        """
        return self._get_wrapped_object_().TrayConfiguration

    @tray_configuration.setter
    def tray_configuration(self, value: str):
        """
        Gets or sets a description of the auto-sampler tray
        """
        assert type(value) is str
        self._get_wrapped_object_().TrayConfiguration = value

    def get_user_column_label(self, index: int) -> str:
        """
        Retrieves the user label at given 0-based label index.
        
        Parameters:
        index:
        Index of user label to be retrieved
        
        Returns:
        String containing the user label at given index
        
        Remarks:
        SampleInformation.MaxUserTextColumnCount determines the maximum number of user
        column labels.
        """
        return self._get_wrapped_object_().GetUserColumnLabel(index)

    def save(self) -> bool:
        """
        Saves Sequence data to disk.
        
        Returns:
        True saved data to disk; false otherwise.
        """
        return self._get_wrapped_object_().Save()

    def set_user_column_label(self, index: int, label: str) -> bool:
        """
        Sets the user label at given 0-based label index.
        
        Parameters:
        index:
        Index of user label to be set
        
        label:
        New string value for user label to be set
        
        Returns:
        true if successful; false otherwise
        
        Remarks:
        SampleInformation.MaxUserTextColumnCount determines the maximum number of user
        column labels.
        """
        return self._get_wrapped_object_().SetUserColumnLabel(index, label)

